@@ -312,10 +312,6 @@ add_preferences_callbacks (void)
 				  "changed::" NAUTILUS_PREFERENCES_SHOW_DIRECTORY_ITEM_COUNTS,
 				  G_CALLBACK (async_data_preference_changed_callback),
 				  NULL);
-	g_signal_connect_swapped (nautilus_preferences,
-				  "changed::" NAUTILUS_PREFERENCES_DATE_FORMAT,
-				  G_CALLBACK(async_data_preference_changed_callback),
-				  NULL);
 }
 
 /**
@@ -542,6 +538,18 @@ nautilus_directory_is_in_trash (NautilusDirectory *directory)
 	return g_file_has_uri_scheme (directory->details->location, "trash");
 }
 
+gboolean
+nautilus_directory_is_in_recent (NautilusDirectory *directory)
+{
+	g_assert (NAUTILUS_IS_DIRECTORY (directory));
+
+	if (directory->details->location == NULL) {
+		return FALSE;
+	}
+
+	return g_file_has_uri_scheme (directory->details->location, "recent");
+}
+
 gboolean
 nautilus_directory_are_all_files_seen (NautilusDirectory *directory)
 {
