@@ -684,7 +684,7 @@ receive_dropped_text (NautilusTreeViewDragDest *dest,
 		      int x, int y)
 {
 	char *drop_target;
-	char *text;
+	guchar *text;
 
 	if (!dest->details->drag_data) {
 		return;
@@ -800,7 +800,7 @@ drag_data_received_callback (GtkWidget *widget,
 			     gpointer data)
 {
 	NautilusTreeViewDragDest *dest;
-	const char *tmp;
+	const gchar *tmp;
 	int length;
 	gboolean success, finished;
 	
@@ -839,7 +839,7 @@ drag_data_received_callback (GtkWidget *widget,
 			break;
 		case NAUTILUS_ICON_DND_RAW:
 			length = gtk_selection_data_get_length (selection_data);
-			tmp = gtk_selection_data_get_data (selection_data);
+			tmp = (const gchar *) gtk_selection_data_get_data (selection_data);
 			receive_dropped_raw (dest, tmp, length, context, x, y);
 			success = TRUE;
 			break;
@@ -888,7 +888,7 @@ get_direct_save_filename (GdkDragContext *context)
 		return NULL;
 	}
 
-	return prop_text;
+	return (gchar *) prop_text;
 }
 
 static gboolean
