@@ -34,6 +34,7 @@
 #include "nautilus-empty-view.h"
 #endif /* ENABLE_EMPTY_VIEW */
 
+#include "nautilus-bookmarks-window.h"
 #include "nautilus-connect-server-dialog.h"
 #include "nautilus-desktop-icon-view.h"
 #include "nautilus-desktop-window.h"
@@ -46,7 +47,6 @@
 #include "nautilus-progress-ui-handler.h"
 #include "nautilus-self-check-functions.h"
 #include "nautilus-window.h"
-#include "nautilus-window-bookmarks.h"
 #include "nautilus-window-manage-views.h"
 #include "nautilus-window-private.h"
 #include "nautilus-window-slot.h"
@@ -108,8 +108,90 @@ struct _NautilusApplicationPriv {
 
 	gboolean no_desktop;
 	gchar *geometry;
+
+	NotifyNotification *unmount_notify;
+
+	GtkWidget *bookmarks_window;
+	NautilusBookmarkList *bookmark_list;
 };
 
+NautilusBookmarkList *
+nautilus_application_get_bookmarks (NautilusApplication *application)
+{
+	return application->priv->bookmark_list;
+}
+
+void
+nautilus_application_edit_bookmarks (NautilusApplication *application,
+				     NautilusWindow      *window)
+{
+	GtkWindow *bookmarks_window;
+
+	bookmarks_window = GTK_WINDOW (application->priv->bookmarks_window);
+
+	if (bookmarks_window == NULL) {
+		bookmarks_window = nautilus_bookmarks_window_new (window, application->priv->bookmark_list);
+		application->priv->bookmarks_window = GTK_WIDGET (bookmarks_window);
+
+		g_object_add_weak_pointer (G_OBJECT (bookmarks_window),
+					   (gpointer *) &application->priv->bookmarks_window);
+	}
+
+	gtk_window_set_transient_for (bookmarks_window, GTK_WINDOW (window));
+	gtk_window_set_screen (bookmarks_window, gtk_window_get_screen (GTK_WINDOW (window)));
+	gtk_window_present (bookmarks_window);
+}
+
+void
+nautilus_application_notify_unmount_done (NautilusApplication *application,
+					  const gchar *message)
+{
+	if (application->priv->unmount_notify) {
+		notify_notification_close (application->priv->unmount_notify, NULL);
+		g_clear_object (&application->priv->unmount_notify);
+	}
+
+	if (message != NULL) {
+		NotifyNotification *unplug;
+		gchar **strings;
+
+		strings = g_strsplit (message, "\n", 0);
+		unplug = notify_notification_new (strings[0], strings[1],
+						  "media-removable");
+
+		notify_notification_show (unplug, NULL);
+		g_object_unref (unplug);
+		g_strfreev (strings);
+	}
+}
+
+void
+nautilus_application_notify_unmount_show (NautilusApplication *application,
+					  const gchar *message)
+{
+	gchar **strings;
+
+	strings = g_strsplit (message, "\n", 0);
+
+	if (!application->priv->unmount_notify) {
+		application->priv->unmount_notify =
+			notify_notification_new (strings[0], strings[1],
+						 "media-removable");
+
+		notify_notification_set_hint (application->priv->unmount_notify,
+					      "transient", g_variant_new_boolean (TRUE));
+		notify_notification_set_urgency (application->priv->unmount_notify,
+						 NOTIFY_URGENCY_CRITICAL);
+	} else {
+		notify_notification_update (application->priv->unmount_notify,
+					    strings[0], strings[1],
+					    "media-removable");
+	}
+
+	notify_notification_show (application->priv->unmount_notify, NULL);
+	g_strfreev (strings);
+}
+
 static gboolean
 check_required_directories (NautilusApplication *application)
 {
@@ -868,6 +950,7 @@ nautilus_application_finalize (GObject *object)
 
 	g_clear_object (&application->priv->volume_monitor);
 	g_clear_object (&application->priv->progress_handler);
+	g_clear_object (&application->priv->bookmark_list);
 
 	g_free (application->priv->geometry);
 
@@ -1301,6 +1384,8 @@ nautilus_application_startup (GApplication *app)
 	g_signal_connect_object (self->priv->volume_monitor, "mount_added",
 				 G_CALLBACK (mount_added_callback), self, 0);
 
+	self->priv->bookmark_list = nautilus_bookmark_list_new ();
+
 	/* Check the user's ~/.nautilus directories and post warnings
 	 * if there are problems.
 	 */
@@ -1320,6 +1405,8 @@ nautilus_application_quit_mainloop (GApplication *app)
 	nautilus_icon_info_clear_caches ();
  	nautilus_application_save_accel_map (NULL);
 
+	nautilus_application_notify_unmount_done (NAUTILUS_APPLICATION (app), NULL);
+
 	G_APPLICATION_CLASS (nautilus_application_parent_class)->quit_mainloop (app);
 }
 
