@@ -46,7 +46,6 @@ typedef enum {
         NORMAL_BUTTON,
         ROOT_BUTTON,
         HOME_BUTTON,
-        DESKTOP_BUTTON,
 	MOUNT_BUTTON
 } ButtonType;
 
@@ -141,32 +140,6 @@ get_slider_button (NautilusPathBar  *path_bar,
         return button;
 }
 
-static void
-trash_state_changed_cb (NautilusTrashMonitor *monitor,
-                        gboolean state,
-                        NautilusPathBar *path_bar)
-{
-        GFile *file;
-        GList *list;
-      
-        file = g_file_new_for_uri ("trash:///");
-        for (list = path_bar->priv->button_list; list; list = list->next) {
-                ButtonData *button_data;
-                button_data = BUTTON_DATA (list->data);
-                if (g_file_equal (file, button_data->path)) {
-                        GIcon *icon;
-                        NautilusIconInfo *icon_info;
-                        GdkPixbuf *pixbuf;
-
-                        icon = nautilus_trash_monitor_get_icon ();
-                        icon_info = nautilus_icon_info_lookup (icon, NAUTILUS_PATH_BAR_ICON_SIZE);                        
-                        pixbuf = nautilus_icon_info_get_pixbuf_at_size (icon_info, NAUTILUS_PATH_BAR_ICON_SIZE);
-                        gtk_image_set_from_pixbuf (GTK_IMAGE (button_data->image), pixbuf);
-                }
-        }
-        g_object_unref (file);
-}
-
 static gboolean
 slider_timeout (gpointer user_data)
 {
@@ -239,7 +212,7 @@ nautilus_path_bar_init (NautilusPathBar *path_bar)
 	gtk_widget_set_has_window (GTK_WIDGET (path_bar), FALSE);
         gtk_widget_set_redraw_on_allocate (GTK_WIDGET (path_bar), FALSE);
 
-        path_bar->priv->spacing = 3;
+        path_bar->priv->spacing = 0;
         path_bar->priv->up_slider_button = get_slider_button (path_bar, GTK_ARROW_LEFT);
         path_bar->priv->down_slider_button = get_slider_button (path_bar, GTK_ARROW_RIGHT);
 
@@ -275,10 +248,8 @@ nautilus_path_bar_init (NautilusPathBar *path_bar)
 			  G_CALLBACK (nautilus_path_bar_slider_drag_leave),
 			  path_bar);
 
-        g_signal_connect (nautilus_trash_monitor_get (),
-                          "trash_state_changed",
-                          G_CALLBACK (trash_state_changed_cb),
-                          path_bar);
+	gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (path_bar)),
+                                     GTK_STYLE_CLASS_LINKED);
 }
 
 static void
@@ -297,9 +268,6 @@ nautilus_path_bar_finalize (GObject *object)
 
         g_list_free (path_bar->priv->button_list);
 
-	g_signal_handlers_disconnect_by_func (nautilus_trash_monitor_get (),
-					      trash_state_changed_cb, path_bar);
-
         G_OBJECT_CLASS (nautilus_path_bar_parent_class)->finalize (object);
 }
 
@@ -329,9 +297,7 @@ nautilus_path_bar_dispose (GObject *object)
 static const char *
 get_dir_name (ButtonData *button_data)
 {
-	if (button_data->type == DESKTOP_BUTTON) {
-		return _("Desktop");
-	} else if (button_data->type == HOME_BUTTON) {
+	if (button_data->type == HOME_BUTTON) {
 		return _("Home");
 	} else {
 		return button_data->dir_name;
@@ -344,11 +310,13 @@ get_dir_name (ButtonData *button_data)
 static void
 set_label_size_request (ButtonData *button_data)
 {
-        const gchar *dir_name = get_dir_name (button_data);
         gint width, height;
-        gchar *markup;
 	GtkRequisition min_req, bold_req;
 
+	if (button_data->label == NULL) {
+		return;
+	}
+
 	gtk_label_set_ellipsize (GTK_LABEL (button_data->label), PANGO_ELLIPSIZE_NONE);
 	gtk_widget_get_preferred_size (button_data->label, &min_req, NULL);
 	gtk_label_set_ellipsize (GTK_LABEL (button_data->label), PANGO_ELLIPSIZE_MIDDLE);	
@@ -478,6 +446,25 @@ nautilus_path_bar_map (GtkWidget *widget)
 	GTK_WIDGET_CLASS (nautilus_path_bar_parent_class)->map (widget);
 }
 
+
+static void
+child_ordering_changed (NautilusPathBar *path_bar)
+{
+	GList *l;
+
+	if (path_bar->priv->up_slider_button) {
+		gtk_style_context_invalidate (gtk_widget_get_style_context (path_bar->priv->up_slider_button));
+	}
+	if (path_bar->priv->down_slider_button) {
+		gtk_style_context_invalidate (gtk_widget_get_style_context (path_bar->priv->down_slider_button));
+	}
+
+	for (l = path_bar->priv->button_list; l; l = l->next) {
+		ButtonData *data = l->data;
+		gtk_style_context_invalidate (gtk_widget_get_style_context (data->button));		
+	}
+}
+
 /* This is a tad complicated */
 static void
 nautilus_path_bar_size_allocate (GtkWidget     *widget,
@@ -494,6 +481,7 @@ nautilus_path_bar_size_allocate (GtkWidget     *widget,
         gint up_slider_offset;
         gint down_slider_offset;
 	GtkRequisition child_requisition;
+	gboolean needs_reorder = FALSE;
 
 	need_sliders = FALSE;
 	up_slider_offset = 0;
@@ -624,7 +612,8 @@ nautilus_path_bar_size_allocate (GtkWidget     *widget,
 			}
 		}
 
-                gtk_widget_set_child_visible (BUTTON_DATA (list->data)->button, TRUE);
+		needs_reorder |= gtk_widget_get_child_visible (child) == FALSE;
+                gtk_widget_set_child_visible (child, TRUE);
                 gtk_widget_size_allocate (child, &child_allocation);
 
                 if (direction == GTK_TEXT_DIR_RTL) {
@@ -637,11 +626,15 @@ nautilus_path_bar_size_allocate (GtkWidget     *widget,
         }
         /* Now we go hide all the widgets that don't fit */
         while (list) {
-        	gtk_widget_set_child_visible (BUTTON_DATA (list->data)->button, FALSE);
+                child = BUTTON_DATA (list->data)->button;
+		needs_reorder |= gtk_widget_get_child_visible (child) == TRUE;
+        	gtk_widget_set_child_visible (child, FALSE);
                 list = list->prev;
         }
         for (list = first_button->next; list; list = list->next) {
- 	        gtk_widget_set_child_visible (BUTTON_DATA (list->data)->button, FALSE);
+                child = BUTTON_DATA (list->data)->button;
+		needs_reorder |= gtk_widget_get_child_visible (child) == TRUE;
+ 	        gtk_widget_set_child_visible (child, FALSE);
         }
 
         if (need_sliders) {
@@ -649,10 +642,12 @@ nautilus_path_bar_size_allocate (GtkWidget     *widget,
                 child_allocation.x = up_slider_offset + allocation->x;
                 gtk_widget_size_allocate (path_bar->priv->up_slider_button, &child_allocation);
 
+		needs_reorder |= gtk_widget_get_child_visible (path_bar->priv->up_slider_button) == FALSE;
                 gtk_widget_set_child_visible (path_bar->priv->up_slider_button, TRUE);
                 gtk_widget_show_all (path_bar->priv->up_slider_button);
 
         } else {
+		needs_reorder |= gtk_widget_get_child_visible (path_bar->priv->up_slider_button) == TRUE;
         	gtk_widget_set_child_visible (path_bar->priv->up_slider_button, FALSE);
         }
 	
@@ -661,12 +656,18 @@ nautilus_path_bar_size_allocate (GtkWidget     *widget,
         	child_allocation.x = down_slider_offset + allocation->x;
         	gtk_widget_size_allocate (path_bar->priv->down_slider_button, &child_allocation);
 
+		needs_reorder |= gtk_widget_get_child_visible (path_bar->priv->down_slider_button) == FALSE;
       		gtk_widget_set_child_visible (path_bar->priv->down_slider_button, TRUE);
       		gtk_widget_show_all (path_bar->priv->down_slider_button);
       		nautilus_path_bar_update_slider_buttons (path_bar);
     	} else {
+		needs_reorder |= gtk_widget_get_child_visible (path_bar->priv->down_slider_button) == TRUE;
     		gtk_widget_set_child_visible (path_bar->priv->down_slider_button, FALSE);
 	}
+
+	if (needs_reorder) {
+		child_ordering_changed (path_bar);
+	}
 }
 
 static void
@@ -864,6 +865,84 @@ nautilus_path_bar_state_changed (GtkWidget    *widget,
 	}
 }
 
+static GtkWidgetPath *
+nautilus_path_bar_get_path_for_child (GtkContainer *container,
+				      GtkWidget    *child)
+{
+	NautilusPathBar *path_bar = NAUTILUS_PATH_BAR (container);
+	GtkWidgetPath *path;
+
+	path = gtk_widget_path_copy (gtk_widget_get_path (GTK_WIDGET (path_bar)));
+
+	if (gtk_widget_get_visible (child) &&
+	    gtk_widget_get_child_visible (child)) {
+		GtkWidgetPath *sibling_path;
+		GList *visible_children;
+		GList *l;
+		int pos;
+
+		/* 1. Build the list of visible children, in visually left-to-right order
+		 * (i.e. independently of the widget's direction).  Note that our
+		 * button_list is stored in innermost-to-outermost path order!
+		 */
+
+		visible_children = NULL;
+
+		if (gtk_widget_get_visible (path_bar->priv->down_slider_button) &&
+		    gtk_widget_get_child_visible (path_bar->priv->down_slider_button)) {
+			visible_children = g_list_prepend (visible_children, path_bar->priv->down_slider_button);
+		}
+
+		for (l = path_bar->priv->button_list; l; l = l->next) {
+			ButtonData *data = l->data;
+				
+			if (gtk_widget_get_visible (data->button) &&
+			    gtk_widget_get_child_visible (data->button))
+				visible_children = g_list_prepend (visible_children, data->button);
+		}
+
+		if (gtk_widget_get_visible (path_bar->priv->up_slider_button) &&
+		    gtk_widget_get_child_visible (path_bar->priv->up_slider_button)) {
+			visible_children = g_list_prepend (visible_children, path_bar->priv->up_slider_button);
+		}
+
+		if (gtk_widget_get_direction (GTK_WIDGET (path_bar)) == GTK_TEXT_DIR_RTL) {
+			visible_children = g_list_reverse (visible_children);
+		}
+
+		/* 2. Find the index of the child within that list */
+
+		pos = 0;
+
+		for (l = visible_children; l; l = l->next) {
+			GtkWidget *button = l->data;
+
+			if (button == child) {
+				break;
+			}
+
+			pos++;
+		}
+
+		/* 3. Build the path */
+
+		sibling_path = gtk_widget_path_new ();
+
+		for (l = visible_children; l; l = l->next) {
+			gtk_widget_path_append_for_widget (sibling_path, l->data);
+		}
+
+		gtk_widget_path_append_with_siblings (path, sibling_path, pos);
+
+		g_list_free (visible_children);
+		gtk_widget_path_unref (sibling_path);
+	} else {
+		gtk_widget_path_append_for_widget (path, child);
+	}
+
+	return path;
+}
+
 static void
 nautilus_path_bar_class_init (NautilusPathBarClass *path_bar_class)
 {
@@ -894,6 +973,7 @@ nautilus_path_bar_class_init (NautilusPathBarClass *path_bar_class)
         container_class->add = nautilus_path_bar_add;
         container_class->forall = nautilus_path_bar_forall;
         container_class->remove = nautilus_path_bar_remove;
+	container_class->get_path_for_child = nautilus_path_bar_get_path_for_child;
 
         path_bar_signals [PATH_CLICKED] =
                 g_signal_new ("path-clicked",
@@ -1216,31 +1296,16 @@ button_drag_begin_cb (GtkWidget *widget,
 			   GINT_TO_POINTER (FALSE));
 }
 
-static NautilusIconInfo *
-get_type_icon_info (ButtonData *button_data)
+static GIcon *
+get_gicon (ButtonData *button_data)
 {
 	switch (button_data->type)
         {
 		case ROOT_BUTTON:
-			return nautilus_icon_info_lookup_from_name (NAUTILUS_ICON_FILESYSTEM,
-								    NAUTILUS_PATH_BAR_ICON_SIZE);
-
+			return g_themed_icon_new (NAUTILUS_ICON_FILESYSTEM);
 		case HOME_BUTTON:
-			return nautilus_icon_info_lookup_from_name (NAUTILUS_ICON_HOME,
-								    NAUTILUS_PATH_BAR_ICON_SIZE);
-
-                case DESKTOP_BUTTON:
-			return nautilus_icon_info_lookup_from_name (NAUTILUS_ICON_DESKTOP,
-								    NAUTILUS_PATH_BAR_ICON_SIZE);
-
-                case NORMAL_BUTTON:
-			if (button_data->is_root) {
-				return nautilus_file_get_icon (button_data->file,
-							       NAUTILUS_PATH_BAR_ICON_SIZE,
-							       NAUTILUS_FILE_ICON_FLAGS_NONE);
-			}
-
-	    	default:
+			return g_themed_icon_new (NAUTILUS_ICON_HOME);
+		default:
 			return NULL;
         }
   
@@ -1268,8 +1333,6 @@ button_data_free (ButtonData *button_data)
 static void
 nautilus_path_bar_update_button_appearance (ButtonData *button_data)
 {
-	NautilusIconInfo *icon_info;
-	GdkPixbuf *pixbuf;
         const gchar *dir_name = get_dir_name (button_data);
 
         if (button_data->label != NULL) {
@@ -1291,19 +1354,13 @@ nautilus_path_bar_update_button_appearance (ButtonData *button_data)
 		gtk_image_set_from_pixbuf (GTK_IMAGE (button_data->image), button_data->custom_icon);  
 		gtk_widget_show (GTK_WIDGET (button_data->image));
 	} else {
-		icon_info = get_type_icon_info (button_data);
+		GIcon *icon;
 
-		pixbuf = NULL;
-
-		if (icon_info != NULL) {
-			pixbuf = nautilus_icon_info_get_pixbuf_at_size (icon_info, NAUTILUS_PATH_BAR_ICON_SIZE);
-			g_object_unref (icon_info);
-		}
-
-		if (pixbuf != NULL) {
-			gtk_image_set_from_pixbuf (GTK_IMAGE (button_data->image), pixbuf);
+		icon = get_gicon (button_data);
+		if (icon != NULL) {
+			gtk_image_set_from_gicon (GTK_IMAGE (button_data->image), icon, GTK_ICON_SIZE_MENU);
 			gtk_widget_show (GTK_WIDGET (button_data->image));
-			g_object_unref (pixbuf);
+			g_object_unref (icon);
 		} else {
 			gtk_widget_hide (GTK_WIDGET (button_data->image));
 		}
@@ -1412,8 +1469,6 @@ setup_button_type (ButtonData       *button_data,
 	} else if (nautilus_is_home_directory (location)) {
 		button_data->type = HOME_BUTTON;
 		button_data->is_root = TRUE;
-	} else if (nautilus_is_desktop_directory (location)) {
-		button_data->type = DESKTOP_BUTTON;
 	} else if (setup_file_path_mounted_mount (location, button_data)) {
 		/* already setup */
 	} else {
@@ -1610,7 +1665,6 @@ make_button_data (NautilusPathBar  *path_bar,
                         button_data->label = NULL;
                         break;
                 case HOME_BUTTON:
-                case DESKTOP_BUTTON:
 		case MOUNT_BUTTON:
 		case NORMAL_BUTTON:
     		default:
@@ -1748,6 +1802,8 @@ nautilus_path_bar_update_path (NautilusPathBar *path_bar,
 	}	
 
         gtk_widget_pop_composite_child ();
+
+	child_ordering_changed (path_bar);
 }
 
 void
