@@ -23,18 +23,17 @@
 
 #include <config.h>
 #include "nautilus-query-editor.h"
-#include "nautilus-window-slot.h"
 
 #include <string.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
-
-#include <eel/eel-glib-extensions.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
+#include <eel/eel-glib-extensions.h>
+#include <libnautilus-private/nautilus-file-utilities.h>
+
 typedef enum {
-	NAUTILUS_QUERY_EDITOR_ROW_LOCATION,
 	NAUTILUS_QUERY_EDITOR_ROW_TYPE,
 	
 	NAUTILUS_QUERY_EDITOR_ROW_LAST
@@ -67,14 +66,15 @@ struct NautilusQueryEditorDetails {
 	gboolean change_frozen;
 	guint typing_timeout_id;
 	gboolean is_visible;
-	GtkWidget *invisible_vbox;
-	GtkWidget *visible_vbox;
+	GtkWidget *vbox;
+
+	GtkWidget *search_current_button;
+	GtkWidget *search_all_button;
+	char *current_uri;
 
 	GList *rows;
 	char *last_set_query_text;
-	
-	NautilusSearchBar *bar;
-	NautilusWindowSlot *slot;
+	gboolean got_preedit;
 };
 
 enum {
@@ -93,12 +93,6 @@ static void nautilus_query_editor_changed (NautilusQueryEditor *editor);
 static NautilusQueryEditorRow * nautilus_query_editor_add_row (NautilusQueryEditor *editor,
 							       NautilusQueryEditorRowType type);
 
-static GtkWidget *location_row_create_widgets  (NautilusQueryEditorRow *row);
-static void       location_row_add_to_query    (NautilusQueryEditorRow *row,
-					        NautilusQuery          *query);
-static void       location_row_free_data       (NautilusQueryEditorRow *row);
-static void       location_add_rows_from_query (NautilusQueryEditor    *editor,
-					        NautilusQuery          *query);
 static GtkWidget *type_row_create_widgets      (NautilusQueryEditorRow *row);
 static void       type_row_add_to_query        (NautilusQueryEditorRow *row,
 					        NautilusQuery          *query);
@@ -109,12 +103,6 @@ static void       type_add_rows_from_query     (NautilusQueryEditor    *editor,
 
 
 static NautilusQueryEditorRowOps row_type[] = {
-	{ N_("Location"),
-	  location_row_create_widgets,
-	  location_row_add_to_query,
-	  location_row_free_data,
-	  location_add_rows_from_query
-	},
 	{ N_("File Type"),
 	  type_row_create_widgets,
 	  type_row_add_to_query,
@@ -125,6 +113,119 @@ static NautilusQueryEditorRowOps row_type[] = {
 
 G_DEFINE_TYPE (NautilusQueryEditor, nautilus_query_editor, GTK_TYPE_BOX);
 
+/* taken from gtk/gtktreeview.c */
+static void
+send_focus_change (GtkWidget *widget,
+                   GdkDevice *device,
+		   gboolean   in)
+{
+	GdkDeviceManager *device_manager;
+	GList *devices, *d;
+
+	device_manager = gdk_display_get_device_manager (gtk_widget_get_display (widget));
+	devices = gdk_device_manager_list_devices (device_manager, GDK_DEVICE_TYPE_MASTER);
+	devices = g_list_concat (devices, gdk_device_manager_list_devices (device_manager, GDK_DEVICE_TYPE_SLAVE));
+	devices = g_list_concat (devices, gdk_device_manager_list_devices (device_manager, GDK_DEVICE_TYPE_FLOATING));
+
+	for (d = devices; d; d = d->next) {
+		GdkDevice *dev = d->data;
+		GdkEvent *fevent;
+		GdkWindow *window;
+
+		if (gdk_device_get_source (dev) != GDK_SOURCE_KEYBOARD)
+			continue;
+
+		window = gtk_widget_get_window (widget);
+
+		/* Skip non-master keyboards that haven't
+		 * selected for events from this window
+		 */
+		if (gdk_device_get_device_type (dev) != GDK_DEVICE_TYPE_MASTER &&
+		    !gdk_window_get_device_events (window, dev))
+			continue;
+
+		fevent = gdk_event_new (GDK_FOCUS_CHANGE);
+
+		fevent->focus_change.type = GDK_FOCUS_CHANGE;
+		fevent->focus_change.window = g_object_ref (window);
+		fevent->focus_change.in = in;
+		gdk_event_set_device (fevent, device);
+
+		gtk_widget_send_focus_change (widget, fevent);
+
+		gdk_event_free (fevent);
+	}
+
+	g_list_free (devices);
+}
+
+static void
+entry_focus_hack (GtkWidget *entry,
+		  GdkDevice *device)
+{
+	GtkEntryClass *entry_class;
+	GtkWidgetClass *entry_parent_class;
+
+	/* Grab focus will select all the text.  We don't want that to happen, so we
+	 * call the parent instance and bypass the selection change.  This is probably
+	 * really non-kosher. */
+	entry_class = g_type_class_peek (GTK_TYPE_ENTRY);
+	entry_parent_class = g_type_class_peek_parent (entry_class);
+	(entry_parent_class->grab_focus) (entry);
+
+	/* send focus-in event */
+	send_focus_change (entry, device, TRUE);
+}
+
+static void
+entry_preedit_changed_cb (GtkEntry            *entry,
+			  gchar               *preedit,
+			  NautilusQueryEditor *editor)
+{
+	editor->details->got_preedit = TRUE;
+}
+
+gboolean
+nautilus_query_editor_handle_event (NautilusQueryEditor *editor,
+				    GdkEventKey         *event)
+{
+	GdkEvent *new_event;
+	gboolean handled = FALSE;
+	gulong id;
+	gboolean retval;
+	gboolean text_changed;
+	char *old_text;
+	const char *new_text;
+
+	editor->details->got_preedit = FALSE;
+	if (!gtk_widget_get_realized (editor->details->entry)) {
+		gtk_widget_realize (editor->details->entry);
+	}
+
+	old_text = g_strdup (gtk_entry_get_text (GTK_ENTRY (editor->details->entry)));
+
+	id = g_signal_connect (editor->details->entry, "preedit-changed",
+			       G_CALLBACK (entry_preedit_changed_cb), editor);
+
+	new_event = gdk_event_copy ((GdkEvent *) event);
+	g_object_unref (((GdkEventKey *) new_event)->window);
+	((GdkEventKey *) new_event)->window = g_object_ref
+		(gtk_widget_get_window (editor->details->entry));
+	retval = gtk_widget_event (editor->details->entry, new_event);
+	gdk_event_free (new_event);
+
+	g_signal_handler_disconnect (editor->details->entry, id);
+
+	new_text = gtk_entry_get_text (GTK_ENTRY (editor->details->entry));
+	text_changed = strcmp (old_text, new_text) != 0;
+	g_free (old_text);
+
+	handled = (editor->details->got_preedit) || (retval && text_changed);
+	editor->details->got_preedit = FALSE;
+
+	return handled;
+}
+
 static void
 nautilus_query_editor_dispose (GObject *object)
 {
@@ -132,26 +233,11 @@ nautilus_query_editor_dispose (GObject *object)
 
 	editor = NAUTILUS_QUERY_EDITOR (object);
 
-	if (editor->details->typing_timeout_id) {
+	if (editor->details->typing_timeout_id > 0) {
 		g_source_remove (editor->details->typing_timeout_id);
 		editor->details->typing_timeout_id = 0;
 	}
 
-	if (editor->details->bar != NULL) {
-		g_signal_handlers_disconnect_by_func (editor->details->entry,
-						      entry_activate_cb,
-						      editor);
-		g_signal_handlers_disconnect_by_func (editor->details->entry,
-						      entry_changed_cb,
-						      editor);
-		
-		nautilus_search_bar_return_entry (editor->details->bar);
-
-		g_object_remove_weak_pointer (G_OBJECT (editor->details->bar),
-					      (gpointer *) &editor->details->bar);
-		editor->details->bar = NULL;
-	}
-
 	G_OBJECT_CLASS (nautilus_query_editor_parent_class)->dispose (object);
 }
 
@@ -176,6 +262,16 @@ nautilus_query_editor_draw (GtkWidget *widget,
 	return FALSE;
 }
 
+static void
+nautilus_query_editor_grab_focus (GtkWidget *widget)
+{
+	NautilusQueryEditor *editor = NAUTILUS_QUERY_EDITOR (widget);
+
+	if (gtk_widget_get_visible (widget)) {
+		entry_focus_hack (editor->details->entry, gtk_get_current_event_device ());
+	}
+}
+
 static void
 nautilus_query_editor_class_init (NautilusQueryEditorClass *class)
 {
@@ -188,6 +284,7 @@ nautilus_query_editor_class_init (NautilusQueryEditorClass *class)
 
 	widget_class = GTK_WIDGET_CLASS (class);
 	widget_class->draw = nautilus_query_editor_draw;
+	widget_class->grab_focus = nautilus_query_editor_grab_focus;
 
 	signals[CHANGED] =
 		g_signal_new ("changed",
@@ -213,10 +310,19 @@ nautilus_query_editor_class_init (NautilusQueryEditorClass *class)
 	g_type_class_add_private (class, sizeof (NautilusQueryEditorDetails));
 }
 
+GFile *
+nautilus_query_editor_get_location (NautilusQueryEditor *editor)
+{
+	GFile *file = NULL;
+	if (editor->details->current_uri != NULL)
+		file = g_file_new_for_uri (editor->details->current_uri);
+	return file;
+}
+
 static void
 entry_activate_cb (GtkWidget *entry, NautilusQueryEditor *editor)
 {
-	if (editor->details->typing_timeout_id) {
+	if (editor->details->typing_timeout_id > 0) {
 		g_source_remove (editor->details->typing_timeout_id);
 		editor->details->typing_timeout_id = 0;
 	}
@@ -230,15 +336,14 @@ typing_timeout_cb (gpointer user_data)
 	NautilusQueryEditor *editor;
 
 	editor = NAUTILUS_QUERY_EDITOR (user_data);
+	editor->details->typing_timeout_id = 0;
 
 	nautilus_query_editor_changed (editor);
 
-	editor->details->typing_timeout_id = 0;
-
 	return FALSE;
 }
 
-#define TYPING_TIMEOUT 750
+#define TYPING_TIMEOUT 250
 
 static void
 entry_changed_cb (GtkWidget *entry, NautilusQueryEditor *editor)
@@ -247,7 +352,7 @@ entry_changed_cb (GtkWidget *entry, NautilusQueryEditor *editor)
 		return;
 	}
 
-	if (editor->details->typing_timeout_id) {
+	if (editor->details->typing_timeout_id > 0) {
 		g_source_remove (editor->details->typing_timeout_id);
 	}
 
@@ -257,88 +362,6 @@ entry_changed_cb (GtkWidget *entry, NautilusQueryEditor *editor)
 			       editor);
 }
 
-static void
-edit_clicked (GtkButton *button, NautilusQueryEditor *editor)
-{
-	nautilus_query_editor_set_visible (editor, TRUE);
-	nautilus_query_editor_grab_focus (editor);
-}
-
-/* Location */
-
-static GtkWidget *
-location_row_create_widgets (NautilusQueryEditorRow *row)
-{
-	GtkWidget *chooser;
-
-	chooser = gtk_file_chooser_button_new (_("Select folder to search in"),
-					       GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
-	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (chooser), TRUE);
-	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
-					     g_get_home_dir ());
-	gtk_widget_show (chooser);
-
-	g_signal_connect_swapped (chooser, "current-folder-changed",
-				  G_CALLBACK (nautilus_query_editor_changed),
-				  row->editor);
-		
-	gtk_box_pack_start (GTK_BOX (row->hbox), chooser, FALSE, FALSE, 0);
-	
-	return chooser;
-}
-
-static void
-location_row_add_to_query (NautilusQueryEditorRow *row,
-			   NautilusQuery          *query)
-{
-	char *folder, *uri;
-	
-	folder = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (row->type_widget));
-	if (folder == NULL) {
-		/* I don't know why, but i got NULL here on initial search in browser mode
-		   even with the location set to the homedir in create_widgets... */
-		folder = g_strdup (g_get_home_dir ());
-	}
-	
-	uri = g_filename_to_uri (folder, NULL, NULL);
-	g_free (folder);
-		
-	nautilus_query_set_location (query, uri);
-	g_free (uri);
-}
-
-static void
-location_row_free_data (NautilusQueryEditorRow *row)
-{
-}
-
-static void
-location_add_rows_from_query (NautilusQueryEditor    *editor,
-			      NautilusQuery          *query)
-{
-	NautilusQueryEditorRow *row;
-	char *uri, *folder;
-	
-	uri = nautilus_query_get_location (query);
-
-	if (uri == NULL) {
-		return;
-	}
-	folder = g_filename_from_uri (uri, NULL, NULL);
-	g_free (uri);
-	if (folder == NULL) {
-		return;
-	}
-	
-	row = nautilus_query_editor_add_row (editor,
-					     NAUTILUS_QUERY_EDITOR_ROW_LOCATION);
-	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (row->type_widget),
-					     folder);
-	
-	g_free (folder);
-}
-
-
 /* Type */
 
 static gboolean
@@ -559,7 +582,8 @@ type_combo_changed (GtkComboBox *combo_box, NautilusQueryEditorRow *row)
 		dialog = gtk_dialog_new_with_buttons (_("Select type"),
 						      GTK_WINDOW (toplevel),
 						      0,
-						      GTK_STOCK_OK, GTK_RESPONSE_OK,
+						      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+						      _("Select"), GTK_RESPONSE_OK,
 						      NULL);
 		gtk_window_set_default_size (GTK_WINDOW (dialog), 400, 600);
 			
@@ -847,7 +871,7 @@ remove_row_cb (GtkButton *clicked_button, NautilusQueryEditorRow *row)
 	NautilusQueryEditor *editor;
 
 	editor = row->editor;
-	gtk_container_remove (GTK_CONTAINER (editor->details->visible_vbox),
+	gtk_container_remove (GTK_CONTAINER (editor->details->vbox),
 			      row->hbox);
 	
 	editor->details->rows = g_list_remove (editor->details->rows, row);
@@ -905,7 +929,7 @@ nautilus_query_editor_add_row (NautilusQueryEditor *editor,
 	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 	row->hbox = hbox;
 	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (editor->details->visible_vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (editor->details->vbox), hbox, FALSE, FALSE, 0);
 
 	combo = gtk_combo_box_text_new ();
 	row->combo = combo;
@@ -925,8 +949,8 @@ nautilus_query_editor_add_row (NautilusQueryEditor *editor,
 	create_type_widgets (row);
 	
 	button = gtk_button_new ();
-	image = gtk_image_new_from_stock (GTK_STOCK_REMOVE,
-					  GTK_ICON_SIZE_SMALL_TOOLBAR);
+	image = gtk_image_new_from_icon_name ("list-remove-symbolic",
+					      GTK_ICON_SIZE_SMALL_TOOLBAR);
 	gtk_container_add (GTK_CONTAINER (button), image);
 	gtk_widget_show (image);
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
@@ -942,12 +966,6 @@ nautilus_query_editor_add_row (NautilusQueryEditor *editor,
 	return row;
 }
 
-static void
-go_search_cb (GtkButton *clicked_button, NautilusQueryEditor *editor)
-{
-	nautilus_query_editor_changed_force (editor, TRUE);
-}
-
 static void
 add_new_row_cb (GtkButton *clicked_button, NautilusQueryEditor *editor)
 {
@@ -958,60 +976,34 @@ add_new_row_cb (GtkButton *clicked_button, NautilusQueryEditor *editor)
 static void
 nautilus_query_editor_init (NautilusQueryEditor *editor)
 {
-	GtkWidget *hbox, *label, *button;
-	char *label_markup;
-
 	editor->details = G_TYPE_INSTANCE_GET_PRIVATE (editor, NAUTILUS_TYPE_QUERY_EDITOR,
 						       NautilusQueryEditorDetails);
-	editor->details->is_visible = TRUE;
 
 	gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (editor)),
-				     GTK_STYLE_CLASS_QUESTION);
+				     GTK_STYLE_CLASS_TOOLBAR);
+	gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (editor)),
+				     GTK_STYLE_CLASS_PRIMARY_TOOLBAR);
+
 	gtk_orientable_set_orientation (GTK_ORIENTABLE (editor), GTK_ORIENTATION_VERTICAL);
 
-	editor->details->invisible_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-	gtk_container_set_border_width (GTK_CONTAINER (editor->details->invisible_vbox), 6);
-	gtk_box_pack_start (GTK_BOX (editor), editor->details->invisible_vbox,
+	editor->details->vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
+	gtk_container_set_border_width (GTK_CONTAINER (editor->details->vbox), 6);
+	gtk_box_pack_start (GTK_BOX (editor), editor->details->vbox,
 			    FALSE, FALSE, 0);
-	editor->details->visible_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-	gtk_container_set_border_width (GTK_CONTAINER (editor->details->visible_vbox), 6);
-	gtk_box_pack_start (GTK_BOX (editor), editor->details->visible_vbox,
-			    FALSE, FALSE, 0);
-	/* Only show visible vbox */
-	gtk_widget_show (editor->details->visible_vbox);
-
-	/* Create invisible part: */
-	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-	gtk_box_pack_start (GTK_BOX (editor->details->invisible_vbox),
-			    hbox, FALSE, FALSE, 0);
-	gtk_widget_show (hbox);
-	
-	label = gtk_label_new ("");
-	label_markup = g_strconcat ("<b>", _("Search Folder"), "</b>", NULL);
-	gtk_label_set_markup (GTK_LABEL (label), label_markup);
-	g_free (label_markup);
-
-	gtk_style_context_add_class (gtk_widget_get_style_context (label),
-				     "nautilus-cluebar-label");
-	
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-	gtk_widget_show (label);
-	
-	button = gtk_button_new_with_label (_("Edit"));
-	gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-	gtk_widget_show (button);
-
-	g_signal_connect (button, "clicked",
-			  G_CALLBACK (edit_clicked), editor);
-
-	gtk_widget_set_tooltip_text (button,
-				     _("Edit the saved search"));
+	gtk_widget_show (editor->details->vbox);
 }
 
-void
-nautilus_query_editor_set_default_query (NautilusQueryEditor *editor)
+static void
+on_all_button_toggled (GtkToggleButton     *button,
+		       NautilusQueryEditor *editor)
+{
+	nautilus_query_editor_changed (editor);
+}
+
+static void
+on_current_button_toggled (GtkToggleButton     *button,
+			   NautilusQueryEditor *editor)
 {
-	nautilus_query_editor_add_row (editor, NAUTILUS_QUERY_EDITOR_ROW_LOCATION);
 	nautilus_query_editor_changed (editor);
 }
 
@@ -1021,8 +1013,8 @@ finish_first_line (NautilusQueryEditor *editor, GtkWidget *hbox, gboolean use_go
 	GtkWidget *button, *image;
 
 	button = gtk_button_new ();
-	image = gtk_image_new_from_stock (GTK_STOCK_ADD,
-					  GTK_ICON_SIZE_SMALL_TOOLBAR);
+	image = gtk_image_new_from_icon_name ("list-add-symbolic",
+					      GTK_ICON_SIZE_SMALL_TOOLBAR);
 	gtk_container_add (GTK_CONTAINER (button), image);
 	gtk_widget_show (image);
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
@@ -1036,46 +1028,32 @@ finish_first_line (NautilusQueryEditor *editor, GtkWidget *hbox, gboolean use_go
 	gtk_widget_set_tooltip_text (button,
 				     _("Add a new criterion to this search"));
 
-	if (use_go) {
-		button = gtk_button_new_with_label (_("Go"));
-	} else {
-		button = gtk_button_new_with_label (_("Reload"));
-	}
-	gtk_widget_show (button);
-
-	gtk_widget_set_tooltip_text (button,
-				     _("Perform or update the search"));
-		
-	g_signal_connect (button, "clicked",
-			  G_CALLBACK (go_search_cb), editor);
-		
-	gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+	editor->details->search_current_button = gtk_radio_button_new_with_label (NULL, _("Current"));
+	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (editor->details->search_current_button), FALSE);
+	gtk_widget_show (editor->details->search_current_button);
+	editor->details->search_all_button = gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (editor->details->search_current_button),
+											  _("All Files"));
+	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (editor->details->search_all_button), FALSE);
+	gtk_widget_show (editor->details->search_all_button);
+	g_signal_connect (editor->details->search_all_button, "toggled",
+			  G_CALLBACK (on_all_button_toggled), editor);
+	g_signal_connect (editor->details->search_current_button, "toggled",
+			  G_CALLBACK (on_current_button_toggled), editor);
+	gtk_box_pack_start (GTK_BOX (hbox), editor->details->search_current_button, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), editor->details->search_all_button, FALSE, FALSE, 0);
 }
 
 static void
-setup_internal_entry (NautilusQueryEditor *editor)
+setup_widgets (NautilusQueryEditor *editor)
 {
-	GtkWidget *hbox, *label;
-	char *label_markup;
-	
+	GtkWidget *hbox;
+
 	/* Create visible part: */
 	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
 	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (editor->details->visible_vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (editor->details->vbox), hbox, FALSE, FALSE, 0);
 
-	label = gtk_label_new ("");
-	label_markup = g_strconcat ("<b>", _("_Search for:"), "</b>", NULL);
-	gtk_label_set_markup_with_mnemonic (GTK_LABEL (label), label_markup);
-	g_free (label_markup);
-	gtk_widget_show (label);
-
-	gtk_style_context_add_class (gtk_widget_get_style_context (label),
-				     "nautilus-cluebar-label");
-	
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-
-	editor->details->entry = gtk_entry_new ();
-	gtk_label_set_mnemonic_widget (GTK_LABEL (label), editor->details->entry);
+	editor->details->entry = gtk_search_entry_new ();
 	gtk_box_pack_start (GTK_BOX (hbox), editor->details->entry, TRUE, TRUE, 0);
 
 	g_signal_connect (editor->details->entry, "activate",
@@ -1087,62 +1065,6 @@ setup_internal_entry (NautilusQueryEditor *editor)
 	finish_first_line (editor, hbox, TRUE);
 }
 
-static void
-setup_external_entry (NautilusQueryEditor *editor, GtkWidget *entry)
-{
-	GtkWidget *hbox, *label;
-	gchar *label_markup;
-	
-	/* Create visible part: */
-	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (editor->details->visible_vbox), hbox, FALSE, FALSE, 0);
-
-	label_markup = g_strconcat ("<b>", _("Search results"), "</b>", NULL);
-	label = gtk_label_new (NULL);
-	gtk_label_set_markup (GTK_LABEL (label), label_markup);
-	gtk_widget_show (label);
-
-	gtk_style_context_add_class (gtk_widget_get_style_context (label),
-				     "nautilus-cluebar-label");
-	
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-	
-	editor->details->entry = entry;
-	g_signal_connect (editor->details->entry, "activate",
-			  G_CALLBACK (entry_activate_cb), editor);
-	g_signal_connect (editor->details->entry, "changed",
-			  G_CALLBACK (entry_changed_cb), editor);
-
-	finish_first_line (editor, hbox, FALSE);
-
-	g_free (label_markup);
-}
-
-void
-nautilus_query_editor_set_visible (NautilusQueryEditor *editor,
-				   gboolean visible)
-{
-	editor->details->is_visible = visible;
-	if (visible) {
-		gtk_widget_show (editor->details->visible_vbox);
-		gtk_widget_hide (editor->details->invisible_vbox);
-	} else {
-		gtk_widget_hide (editor->details->visible_vbox);
-		gtk_widget_show (editor->details->invisible_vbox);
-	}
-}
-
-static gboolean
-query_is_valid (NautilusQueryEditor *editor)
-{
-	const char *text;
-
-	text = gtk_entry_get_text (GTK_ENTRY (editor->details->entry));
-
-	return text != NULL && text[0] != '\0';
-}
-
 static void
 nautilus_query_editor_changed_force (NautilusQueryEditor *editor, gboolean force_reload)
 {
@@ -1151,27 +1073,33 @@ nautilus_query_editor_changed_force (NautilusQueryEditor *editor, gboolean force
 	if (editor->details->change_frozen) {
 		return;
 	}
-	
-	if (query_is_valid (editor)) {
-		query = nautilus_query_editor_get_query (editor);
-		g_signal_emit (editor, signals[CHANGED], 0,
-			       query, force_reload);
-		g_object_unref (query);
-	}
+
+	query = nautilus_query_editor_get_query (editor);
+	g_signal_emit (editor, signals[CHANGED], 0,
+		       query, force_reload);
+	g_object_unref (query);
 }
 
 static void
 nautilus_query_editor_changed (NautilusQueryEditor *editor)
 {
-	nautilus_query_editor_changed_force (editor, FALSE);
+	nautilus_query_editor_changed_force (editor, TRUE);
 }
 
-void
-nautilus_query_editor_grab_focus (NautilusQueryEditor *editor)
+static void
+add_location_to_query (NautilusQueryEditor *editor,
+		       NautilusQuery       *query)
 {
-	if (editor->details->is_visible) {
-		gtk_widget_grab_focus (editor->details->entry);
+	char *uri;
+
+	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (editor->details->search_all_button))) {
+		uri = nautilus_get_home_directory_uri ();
+	} else {
+		uri = g_strdup (editor->details->current_uri);
 	}
+
+	nautilus_query_set_location (query, uri);
+	g_free (uri);
 }
 
 NautilusQuery *
@@ -1188,14 +1116,11 @@ nautilus_query_editor_get_query (NautilusQueryEditor *editor)
 
 	query_text = gtk_entry_get_text (GTK_ENTRY (editor->details->entry));
 
-	/* Empty string is a NULL query */
-	if (query_text && query_text[0] == '\0') {
-		return NULL;
-	}
-	
 	query = nautilus_query_new ();
 	nautilus_query_set_text (query, query_text);
 
+	add_location_to_query (editor, query);
+
 	for (l = editor->details->rows; l != NULL; l = l->next) {
 		row = l->data;
 		
@@ -1205,112 +1130,62 @@ nautilus_query_editor_get_query (NautilusQueryEditor *editor)
 	return query;
 }
 
-void
-nautilus_query_editor_clear_query (NautilusQueryEditor *editor)
-{
-	editor->details->change_frozen = TRUE;
-	gtk_entry_set_text (GTK_ENTRY (editor->details->entry), "");
-
-	g_free (editor->details->last_set_query_text);
-	editor->details->last_set_query_text = g_strdup ("");
-
-	editor->details->change_frozen = FALSE;
-}
-
 GtkWidget *
-nautilus_query_editor_new (gboolean start_hidden)
+nautilus_query_editor_new (void)
 {
 	GtkWidget *editor;
 
 	editor = g_object_new (NAUTILUS_TYPE_QUERY_EDITOR, NULL);
-	nautilus_query_editor_set_visible (NAUTILUS_QUERY_EDITOR (editor),
-					   !start_hidden);
-	
-	setup_internal_entry (NAUTILUS_QUERY_EDITOR (editor));
-		
+	setup_widgets (NAUTILUS_QUERY_EDITOR (editor));
+
 	return editor;
 }
 
 static void
-detach_from_external_entry (NautilusQueryEditor *editor)
+update_location (NautilusQueryEditor *editor)
 {
-	if (editor->details->bar != NULL) {
-		nautilus_search_bar_return_entry (editor->details->bar);
-		g_signal_handlers_block_by_func (editor->details->entry,
-						 entry_activate_cb,
-						 editor);
-		g_signal_handlers_block_by_func (editor->details->entry,
-						 entry_changed_cb,
-						 editor);
+	NautilusFile *file;
+
+	file = nautilus_file_get_by_uri (editor->details->current_uri);
+
+	if (file != NULL) {
+		char *name;
+		if (nautilus_file_is_home (file)) {
+			name = g_strdup (_("Home"));
+		} else {
+			char *filename;
+			filename = nautilus_file_get_display_name (file);
+			name = g_strdup_printf ("\342\200\234%s\342\200\235", filename);
+			g_free (filename);
+		}
+		gtk_button_set_label (GTK_BUTTON (editor->details->search_current_button),
+				      name);
+		g_free (name);
+
+		nautilus_file_unref (file);
 	}
 }
 
-static void
-attach_to_external_entry (NautilusQueryEditor *editor)
-{
-	if (editor->details->bar != NULL) {
-		nautilus_search_bar_borrow_entry (editor->details->bar);
-		g_signal_handlers_unblock_by_func (editor->details->entry,
-						   entry_activate_cb,
-						   editor);
-		g_signal_handlers_unblock_by_func (editor->details->entry,
-						   entry_changed_cb,
-						   editor);
-
-		editor->details->change_frozen = TRUE;
-		gtk_entry_set_text (GTK_ENTRY (editor->details->entry),
-				    editor->details->last_set_query_text);
-		editor->details->change_frozen = FALSE;
-	}
-}
-
-GtkWidget*
-nautilus_query_editor_new_with_bar (gboolean start_hidden,
-				    gboolean start_attached,
-				    NautilusSearchBar *bar,
-				    NautilusWindowSlot *slot)
-{
-	GtkWidget *entry;
-	NautilusQueryEditor *editor;
-
-	editor = NAUTILUS_QUERY_EDITOR (g_object_new (NAUTILUS_TYPE_QUERY_EDITOR, NULL));
-	nautilus_query_editor_set_visible (editor, !start_hidden);
-
-	editor->details->bar = bar;
-	g_object_add_weak_pointer (G_OBJECT (editor->details->bar),
-				   (gpointer *) &editor->details->bar);
-
-	editor->details->slot = slot;
-
-	entry = nautilus_search_bar_borrow_entry (bar);
-	setup_external_entry (editor, entry);
-	if (!start_attached) {
-		detach_from_external_entry (editor);
-	}
-
-	g_signal_connect_object (slot, "active",
-				 G_CALLBACK (attach_to_external_entry),
-				 editor, G_CONNECT_SWAPPED);
-	g_signal_connect_object (slot, "inactive",
-				 G_CALLBACK (detach_from_external_entry),
-				 editor, G_CONNECT_SWAPPED);
-	
-	return GTK_WIDGET (editor);
-}
-
 void
-nautilus_query_editor_set_query (NautilusQueryEditor *editor, NautilusQuery *query)
+nautilus_query_editor_set_location (NautilusQueryEditor *editor,
+				    GFile               *location)
+{
+	g_free (editor->details->current_uri);
+	editor->details->current_uri = g_file_get_uri (location);
+	update_location (editor);
+}
+
+void
+nautilus_query_editor_set_query (NautilusQueryEditor	*editor,
+				 NautilusQuery		*query)
 {
 	NautilusQueryEditorRowType type;
-	char *text;
+	char *text = NULL;
 
-	if (!query) {
-		nautilus_query_editor_clear_query (editor);
-		return;
+	if (query != NULL) {
+		text = nautilus_query_get_text (query);
 	}
 
-	text = nautilus_query_get_text (query);
-
 	if (!text) {
 		text = g_strdup ("");
 	}
@@ -1318,12 +1193,21 @@ nautilus_query_editor_set_query (NautilusQueryEditor *editor, NautilusQuery *que
 	editor->details->change_frozen = TRUE;
 	gtk_entry_set_text (GTK_ENTRY (editor->details->entry), text);
 
-	for (type = 0; type < NAUTILUS_QUERY_EDITOR_ROW_LAST; type++) {
-		row_type[type].add_rows_from_query (editor, query);
+	g_free (editor->details->current_uri);
+	editor->details->current_uri = NULL;
+
+	if (query != NULL) {
+		editor->details->current_uri = nautilus_query_get_location (query);
+		update_location (editor);
+
+
+		for (type = 0; type < NAUTILUS_QUERY_EDITOR_ROW_LAST; type++) {
+			row_type[type].add_rows_from_query (editor, query);
+		}
 	}
-	
-	editor->details->change_frozen = FALSE;
 
 	g_free (editor->details->last_set_query_text);
 	editor->details->last_set_query_text = text;
+
+	editor->details->change_frozen = FALSE;
 }
