@@ -29,11 +29,14 @@
 
 #include "nautilus-location-bar.h"
 #include "nautilus-pathbar.h"
+#include "nautilus-actions.h"
 #include "nautilus-window-private.h"
 
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-ui-utilities.h>
 
+#include <math.h>
+
 struct _NautilusToolbarPriv {
 	GtkWidget *toolbar;
 
@@ -42,18 +45,16 @@ struct _NautilusToolbarPriv {
 
 	GtkWidget *path_bar;
 	GtkWidget *location_bar;
-	GtkWidget *search_bar;
 
-	gboolean show_main_bar;
+	GtkToolItem *back_forward;
+
 	gboolean show_location_entry;
-	gboolean show_search_bar;
 };
 
 enum {
 	PROP_ACTION_GROUP = 1,
+	PROP_UI_MANAGER,
 	PROP_SHOW_LOCATION_ENTRY,
-	PROP_SHOW_SEARCH_BAR,
-	PROP_SHOW_MAIN_BAR,
 	NUM_PROPERTIES
 };
 
@@ -69,54 +70,115 @@ toolbar_update_appearance (NautilusToolbar *self)
 	show_location_entry = self->priv->show_location_entry ||
 		g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY);
 
-	gtk_widget_set_visible (self->priv->toolbar,
-				self->priv->show_main_bar);
-
 	gtk_widget_set_visible (self->priv->location_bar,
 				show_location_entry);
 	gtk_widget_set_visible (self->priv->path_bar,
 				!show_location_entry);
+}
 
-	gtk_widget_set_visible (self->priv->search_bar,
-				self->priv->show_search_bar);
+static gint
+get_icon_margin (NautilusToolbar *self)
+{
+	GtkIconSize toolbar_size;
+	gint toolbar_size_px, menu_size_px;
+
+	toolbar_size = gtk_toolbar_get_icon_size (GTK_TOOLBAR (self->priv->toolbar));
+
+	gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &menu_size_px, NULL);
+	gtk_icon_size_lookup (toolbar_size, &toolbar_size_px, NULL);
+
+	return (gint) floor ((toolbar_size_px - menu_size_px) / 2.0);
+}
+
+static GtkWidget *
+toolbar_create_toolbutton (NautilusToolbar *self,
+			   gboolean create_menu,
+			   gboolean create_toggle,
+			   const gchar *name)
+{
+	GtkWidget *button, *image;
+	GtkAction *action;
+
+	if (create_menu) {
+		button = gtk_menu_button_new ();
+	} else if (create_toggle) {
+		button = gtk_toggle_button_new ();
+	} else {
+		button = gtk_button_new ();
+	}
+
+	image = gtk_image_new ();
+	g_object_set (image, "margin", get_icon_margin (self), NULL);
+
+	gtk_button_set_image (GTK_BUTTON (button), image);
+
+	if (create_menu) {
+		gtk_image_set_from_icon_name (GTK_IMAGE (image), "emblem-system-symbolic",
+					      GTK_ICON_SIZE_MENU);
+	} else {
+		action = gtk_action_group_get_action (self->priv->action_group, name);
+		gtk_activatable_set_related_action (GTK_ACTIVATABLE (button), action);
+		gtk_button_set_label (GTK_BUTTON (button), NULL);
+	}
+
+	return button;
 }
 
 static void
 nautilus_toolbar_constructed (GObject *obj)
 {
 	NautilusToolbar *self = NAUTILUS_TOOLBAR (obj);
-	GtkToolItem *item;
-	GtkWidget *hbox, *toolbar, *search;
+	GtkWidget *hbox, *toolbar;
 	GtkStyleContext *context;
+	GtkWidget *tool_button;
+	GtkWidget *menu;
+	GtkWidget *box;
+	GtkToolItem *back_forward;
+	GtkToolItem *tool_item;
 
 	G_OBJECT_CLASS (nautilus_toolbar_parent_class)->constructed (obj);
 
 	gtk_style_context_set_junction_sides (gtk_widget_get_style_context (GTK_WIDGET (self)),
 					      GTK_JUNCTION_BOTTOM);
 
-	/* add the UI */
-	self->priv->ui_manager = gtk_ui_manager_new ();
-	gtk_ui_manager_add_ui_from_resource (self->priv->ui_manager, "/org/gnome/nautilus/nautilus-toolbar-ui.xml", NULL);
-	gtk_ui_manager_insert_action_group (self->priv->ui_manager, self->priv->action_group, 0);
-
-	toolbar = gtk_ui_manager_get_widget (self->priv->ui_manager, "/Toolbar");
+	toolbar = gtk_toolbar_new ();
 	self->priv->toolbar = toolbar;
-	gtk_toolbar_set_icon_size (GTK_TOOLBAR (toolbar), GTK_ICON_SIZE_SMALL_TOOLBAR);
-
-	context = gtk_widget_get_style_context (toolbar);
-	gtk_style_context_add_class (context, GTK_STYLE_CLASS_PRIMARY_TOOLBAR);
-
-	search = gtk_ui_manager_get_widget (self->priv->ui_manager, "/Toolbar/Search");
-	gtk_style_context_add_class (gtk_widget_get_style_context (search), GTK_STYLE_CLASS_RAISED);
-	gtk_widget_set_name (search, "nautilus-search-button");
 
 	gtk_box_pack_start (GTK_BOX (self), self->priv->toolbar, TRUE, TRUE, 0);
 	gtk_widget_show_all (self->priv->toolbar);
 
+	context = gtk_widget_get_style_context (toolbar);
+	/* Set the MENUBAR style class so it's possible to drag the app
+	 * using the toolbar. */
+	gtk_style_context_add_class (context, GTK_STYLE_CLASS_MENUBAR);
+
+	/* Back and Forward */
+	back_forward = gtk_tool_item_new ();
+	box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+
+	/* Back */
+	tool_button = toolbar_create_toolbutton (self, FALSE, FALSE, NAUTILUS_ACTION_BACK);
+	gtk_container_add (GTK_CONTAINER (box), GTK_WIDGET (tool_button));
+
+	/* Forward */
+	tool_button = toolbar_create_toolbutton (self, FALSE, FALSE, NAUTILUS_ACTION_FORWARD);
+	gtk_container_add (GTK_CONTAINER (box), GTK_WIDGET (tool_button));
+
+	gtk_style_context_add_class (gtk_widget_get_style_context (box),
+				     GTK_STYLE_CLASS_RAISED);
+	gtk_style_context_add_class (gtk_widget_get_style_context (box),
+				     GTK_STYLE_CLASS_LINKED);
+
+	gtk_container_add (GTK_CONTAINER (back_forward), box);
+	gtk_container_add (GTK_CONTAINER (self->priv->toolbar), GTK_WIDGET (back_forward));
+
+	gtk_widget_show_all (GTK_WIDGET (back_forward));
+	gtk_widget_set_margin_right (GTK_WIDGET (back_forward), 12);
+
+	/* regular path bar */
 	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 	gtk_widget_show (hbox);
 
-	/* regular path bar */
 	self->priv->path_bar = g_object_new (NAUTILUS_TYPE_PATH_BAR, NULL);
 	gtk_box_pack_start (GTK_BOX (hbox), self->priv->path_bar, TRUE, TRUE, 0);
 
@@ -124,15 +186,29 @@ nautilus_toolbar_constructed (GObject *obj)
 	self->priv->location_bar = nautilus_location_bar_new ();
 	gtk_box_pack_start (GTK_BOX (hbox), self->priv->location_bar, TRUE, TRUE, 0);
 
-	item = gtk_tool_item_new ();
-	gtk_tool_item_set_expand (item, TRUE);
-	gtk_container_add (GTK_CONTAINER (item), hbox);
-	gtk_toolbar_insert (GTK_TOOLBAR (self->priv->toolbar), item, 0);
-	gtk_widget_show (GTK_WIDGET (item));
+	tool_item = gtk_tool_item_new ();
+	gtk_tool_item_set_expand (tool_item, TRUE);
+	gtk_container_add (GTK_CONTAINER (tool_item), hbox);
+	gtk_container_add (GTK_CONTAINER (self->priv->toolbar), GTK_WIDGET (tool_item));
+	gtk_widget_show (GTK_WIDGET (tool_item));
 
-	/* search bar */
-	self->priv->search_bar = nautilus_search_bar_new ();
-	gtk_box_pack_start (GTK_BOX (self), self->priv->search_bar, TRUE, TRUE, 0);
+	/* search */
+	tool_item = gtk_tool_item_new ();
+	tool_button = toolbar_create_toolbutton (self, FALSE, TRUE, NAUTILUS_ACTION_SEARCH);
+	gtk_container_add (GTK_CONTAINER (tool_item), GTK_WIDGET (tool_button));
+	gtk_container_add (GTK_CONTAINER (self->priv->toolbar), GTK_WIDGET (tool_item));
+	gtk_widget_show_all (GTK_WIDGET (tool_item));
+	gtk_widget_set_margin_left (GTK_WIDGET (tool_item), 12);
+
+	/* Page Menu */
+	tool_item = gtk_tool_item_new ();
+	tool_button = toolbar_create_toolbutton (self, TRUE, FALSE, "emblem-system-symbolic");
+	menu = gtk_ui_manager_get_widget (self->priv->ui_manager, "/ViewMenu");
+	gtk_menu_button_set_menu (GTK_MENU_BUTTON (tool_button), menu);
+	gtk_container_add (GTK_CONTAINER (tool_item), tool_button);
+	gtk_container_add (GTK_CONTAINER (toolbar), GTK_WIDGET (tool_item));
+	gtk_widget_show_all (GTK_WIDGET (tool_item));
+	gtk_widget_set_margin_left (GTK_WIDGET (tool_item), 6);
 
 	g_signal_connect_swapped (nautilus_preferences,
 				  "changed::" NAUTILUS_PREFERENCES_ALWAYS_USE_LOCATION_ENTRY,
@@ -146,7 +222,6 @@ nautilus_toolbar_init (NautilusToolbar *self)
 {
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, NAUTILUS_TYPE_TOOLBAR,
 						  NautilusToolbarPriv);
-	self->priv->show_main_bar = TRUE;	
 }
 
 static void
@@ -161,12 +236,6 @@ nautilus_toolbar_get_property (GObject *object,
 	case PROP_SHOW_LOCATION_ENTRY:
 		g_value_set_boolean (value, self->priv->show_location_entry);
 		break;
-	case PROP_SHOW_SEARCH_BAR:
-		g_value_set_boolean (value, self->priv->show_search_bar);
-		break;
-	case PROP_SHOW_MAIN_BAR:
-		g_value_set_boolean (value, self->priv->show_main_bar);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -182,18 +251,15 @@ nautilus_toolbar_set_property (GObject *object,
 	NautilusToolbar *self = NAUTILUS_TOOLBAR (object);
 
 	switch (property_id) {
+	case PROP_UI_MANAGER:
+		self->priv->ui_manager = g_value_get_object (value);
+		break;
 	case PROP_ACTION_GROUP:
 		self->priv->action_group = g_value_dup_object (value);
 		break;
 	case PROP_SHOW_LOCATION_ENTRY:
 		nautilus_toolbar_set_show_location_entry (self, g_value_get_boolean (value));
 		break;
-	case PROP_SHOW_SEARCH_BAR:
-		nautilus_toolbar_set_show_search_bar (self, g_value_get_boolean (value));
-		break;
-	case PROP_SHOW_MAIN_BAR:
-		nautilus_toolbar_set_show_main_bar (self, g_value_get_boolean (value));
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
 		break;
@@ -205,9 +271,7 @@ nautilus_toolbar_dispose (GObject *obj)
 {
 	NautilusToolbar *self = NAUTILUS_TOOLBAR (obj);
 
-	g_clear_object (&self->priv->ui_manager);
 	g_clear_object (&self->priv->action_group);
-
 	g_signal_handlers_disconnect_by_func (nautilus_preferences,
 					      toolbar_update_appearance, self);
 
@@ -232,34 +296,31 @@ nautilus_toolbar_class_init (NautilusToolbarClass *klass)
 				     GTK_TYPE_ACTION_GROUP,
 				     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY |
 				     G_PARAM_STATIC_STRINGS);
+	properties[PROP_UI_MANAGER] =
+		g_param_spec_object ("ui-manager",
+				     "The UI manager",
+				     "The UI manager",
+				     GTK_TYPE_UI_MANAGER,
+				     G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY |
+				     G_PARAM_STATIC_STRINGS);
 	properties[PROP_SHOW_LOCATION_ENTRY] =
 		g_param_spec_boolean ("show-location-entry",
 				      "Whether to show the location entry",
 				      "Whether to show the location entry instead of the pathbar",
 				      FALSE,
 				      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-	properties[PROP_SHOW_SEARCH_BAR] =
-		g_param_spec_boolean ("show-search-bar",
-				      "Whether to show the search bar",
-				      "Whether to show the search bar beside the toolbar",
-				      FALSE,
-				      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
-	properties[PROP_SHOW_MAIN_BAR] =
-		g_param_spec_boolean ("show-main-bar",
-				      "Whether to show the main bar",
-				      "Whether to show the main toolbar",
-				      TRUE,
-				      G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 	
 	g_type_class_add_private (klass, sizeof (NautilusToolbarClass));
 	g_object_class_install_properties (oclass, NUM_PROPERTIES, properties);
 }
 
 GtkWidget *
-nautilus_toolbar_new (GtkActionGroup *action_group)
+nautilus_toolbar_new (GtkUIManager *ui_manager,
+		      GtkActionGroup *action_group)
 {
 	return g_object_new (NAUTILUS_TYPE_TOOLBAR,
 			     "action-group", action_group,
+			     "ui-manager", ui_manager,
 			     "orientation", GTK_ORIENTATION_VERTICAL,
 			     NULL);
 }
@@ -276,24 +337,6 @@ nautilus_toolbar_get_location_bar (NautilusToolbar *self)
 	return self->priv->location_bar;
 }
 
-GtkWidget *
-nautilus_toolbar_get_search_bar (NautilusToolbar *self)
-{
-	return self->priv->search_bar;
-}
-
-void
-nautilus_toolbar_set_show_main_bar (NautilusToolbar *self,
-				    gboolean show_main_bar)
-{
-	if (show_main_bar != self->priv->show_main_bar) {
-		self->priv->show_main_bar = show_main_bar;
-		toolbar_update_appearance (self);
-
-		g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SHOW_MAIN_BAR]);
-	}
-}
-
 void
 nautilus_toolbar_set_show_location_entry (NautilusToolbar *self,
 					  gboolean show_location_entry)
@@ -305,15 +348,3 @@ nautilus_toolbar_set_show_location_entry (NautilusToolbar *self,
 		g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SHOW_LOCATION_ENTRY]);
 	}
 }
-
-void
-nautilus_toolbar_set_show_search_bar (NautilusToolbar *self,
-				      gboolean show_search_bar)
-{
-	if (show_search_bar != self->priv->show_search_bar) {
-		self->priv->show_search_bar = show_search_bar;
-		toolbar_update_appearance (self);
-
-		g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_SHOW_SEARCH_BAR]);
-	}
-}
