@@ -36,10 +36,8 @@
 #include "nautilus-navigation-action.h"
 #include "nautilus-notebook.h"
 #include "nautilus-window-manage-views.h"
-#include "nautilus-window-bookmarks.h"
 #include "nautilus-window-private.h"
 #include "nautilus-desktop-window.h"
-#include "nautilus-search-bar.h"
 #include <gtk/gtk.h>
 #include <gio/gio.h>
 #include <glib/gi18n.h>
@@ -60,7 +58,7 @@
 #include <libnautilus-private/nautilus-trash-monitor.h>
 #include <string.h>
 
-#define MENU_PATH_EXTENSION_ACTIONS                     "/MenuBar/File/Extension Actions"
+#define MENU_PATH_EXTENSION_ACTIONS                     "/ViewMenu/Extension Actions"
 #define POPUP_PATH_EXTENSION_ACTIONS                     "/background/Before Zoom Items/Extension Actions"
 
 #define NETWORK_URI          "network:"
@@ -113,60 +111,6 @@ action_home_callback (GtkAction *action,
 				      nautilus_event_get_window_open_flags ());
 }
 
-static void
-action_go_to_network_callback (GtkAction *action, 
-				gpointer user_data) 
-{
-	NautilusWindow *window;
-	NautilusWindowSlot *slot;
-	GFile *network;
-
-	window = NAUTILUS_WINDOW (user_data);
-	slot = nautilus_window_get_active_slot (window);
-
-	network = g_file_new_for_uri (NETWORK_URI);
-	nautilus_window_slot_open_location (slot, network,
-					    nautilus_event_get_window_open_flags ());
-	g_object_unref (network);
-}
-
-static void
-action_go_to_templates_callback (GtkAction *action,
-				 gpointer user_data) 
-{
-	NautilusWindow *window;
-	NautilusWindowSlot *slot;
-	char *path;
-	GFile *location;
-
-	window = NAUTILUS_WINDOW (user_data);
-	slot = nautilus_window_get_active_slot (window);
-
-	path = nautilus_get_templates_directory ();
-	location = g_file_new_for_path (path);
-	g_free (path);
-	nautilus_window_slot_open_location (slot, location,
-					    nautilus_event_get_window_open_flags ());
-	g_object_unref (location);
-}
-
-static void
-action_go_to_trash_callback (GtkAction *action, 
-			     gpointer user_data) 
-{
-	NautilusWindow *window;
-	NautilusWindowSlot *slot;
-	GFile *trash;
-
-	window = NAUTILUS_WINDOW (user_data);
-	slot = nautilus_window_get_active_slot (window);
-
-	trash = g_file_new_for_uri ("trash:///");
-	nautilus_window_slot_open_location (slot, trash,
-					    nautilus_event_get_window_open_flags ());
-	g_object_unref (trash);
-}
-
 static void
 action_reload_callback (GtkAction *action, 
 			gpointer user_data) 
@@ -330,39 +274,6 @@ action_nautilus_manual_callback (GtkAction *action,
 	}
 }
 
-static void
-trash_state_changed_cb (NautilusTrashMonitor *monitor,
-			gboolean state,
-			NautilusWindow *window)
-{
-	GtkActionGroup *action_group;
-	GtkAction *action;
-	GIcon *gicon;
-
-	action_group = nautilus_window_get_main_action_group (window);
-	action = gtk_action_group_get_action (action_group, "Go to Trash");
-
-	gicon = nautilus_trash_monitor_get_icon ();
-
-	if (gicon) {
-		g_object_set (action, "gicon", gicon, NULL);
-		g_object_unref (gicon);
-	}
-}
-
-static void
-nautilus_window_initialize_trash_icon_monitor (NautilusWindow *window)
-{
-	NautilusTrashMonitor *monitor;
-
-	monitor = nautilus_trash_monitor_get ();
-
-	trash_state_changed_cb (monitor, TRUE, window);
-
-	g_signal_connect (monitor, "trash_state_changed",
-			  G_CALLBACK (trash_state_changed_cb), window);
-}
-
 #define MENU_ITEM_MAX_WIDTH_CHARS 32
 
 static void
@@ -408,14 +319,27 @@ static void
 action_add_bookmark_callback (GtkAction *action,
 			      gpointer user_data)
 {
-        nautilus_window_add_bookmark_for_current_location (NAUTILUS_WINDOW (user_data));
+	NautilusWindow *window = user_data;
+	NautilusApplication *app = NAUTILUS_APPLICATION (g_application_get_default ());
+	NautilusBookmark *bookmark;
+	NautilusWindowSlot *slot;
+	NautilusBookmarkList *list;
+
+	slot = nautilus_window_get_active_slot (window);
+	bookmark = slot->current_location_bookmark;
+	list = nautilus_application_get_bookmarks (app);
+
+	if (!nautilus_bookmark_list_contains (list, bookmark)) {
+		nautilus_bookmark_list_append (list, bookmark);
+	}
 }
 
 static void
 action_edit_bookmarks_callback (GtkAction *action, 
 				gpointer user_data)
 {
-        nautilus_window_edit_bookmarks (NAUTILUS_WINDOW (user_data));
+	NautilusApplication *app = NAUTILUS_APPLICATION (g_application_get_default ());
+	nautilus_application_edit_bookmarks (app, NAUTILUS_WINDOW (user_data));
 }
 
 static void
@@ -435,39 +359,6 @@ connect_proxy_cb (GtkActionGroup *action_group,
 	gtk_label_set_max_width_chars (label, MENU_ITEM_MAX_WIDTH_CHARS);
 }
 
-static const char* icon_entries[] = {
-	"/MenuBar/Other Menus/Go/Home",
-	"/MenuBar/Other Menus/Go/Go to Templates",
-	"/MenuBar/Other Menus/Go/Go to Trash",
-	"/MenuBar/Other Menus/Go/Go to Network",
-	"/MenuBar/Other Menus/Go/Go to Location"
-};
-
-/**
- * refresh_go_menu:
- * 
- * Refresh list of bookmarks at end of Go menu to match centralized history list.
- * @window: The NautilusWindow whose Go menu will be refreshed.
- **/
-static void
-nautilus_window_initialize_go_menu (NautilusWindow *window)
-{
-	GtkUIManager *ui_manager;
-	GtkWidget *menuitem;
-	int i;
-
-	ui_manager = nautilus_window_get_ui_manager (NAUTILUS_WINDOW (window));
-
-	for (i = 0; i < G_N_ELEMENTS (icon_entries); i++) {
-		menuitem = gtk_ui_manager_get_widget (
-				ui_manager,
-				icon_entries[i]);
-
-		gtk_image_menu_item_set_always_show_image (
-				GTK_IMAGE_MENU_ITEM (menuitem), TRUE);
-	}
-}
-
 static void
 action_new_window_callback (GtkAction *action,
 			    gpointer user_data)
@@ -629,18 +520,6 @@ static const GtkActionEntry main_entries[] = {
   /* label, accelerator */       N_("_Home"), "<alt>Home",
   /* tooltip */                  N_("Open your personal folder"),
                                  G_CALLBACK (action_home_callback) },
-  /* name, stock id */         { "Go to Network", NAUTILUS_ICON_NETWORK,
-  /* label, accelerator */       N_("_Network"), NULL,
-  /* tooltip */                  N_("Browse bookmarked and local network locations"),
-                                 G_CALLBACK (action_go_to_network_callback) },
-  /* name, stock id */         { "Go to Templates", NAUTILUS_ICON_TEMPLATE,
-  /* label, accelerator */       N_("T_emplates"), NULL,
-  /* tooltip */                  N_("Open your personal templates folder"),
-                                 G_CALLBACK (action_go_to_templates_callback) },
-  /* name, stock id */         { "Go to Trash", NAUTILUS_ICON_TRASH,
-  /* label, accelerator */       N_("_Trash"), NULL,
-  /* tooltip */                  N_("Open your personal trash folder"),
-                                 G_CALLBACK (action_go_to_trash_callback) },
   /* name, stock id, label */  { "Go", NULL, N_("_Go") },
   /* name, stock id, label */  { "Bookmarks", NULL, N_("_Bookmarks") },
   /* name, stock id, label */  { "Tabs", NULL, N_("_Tabs") },
@@ -653,10 +532,10 @@ static const GtkActionEntry main_entries[] = {
   /* name, stock id, label */  { NAUTILUS_ACTION_CLOSE_ALL_WINDOWS, NULL, N_("Close _All Windows"),
                                  "<control>Q", N_("Close all Navigation windows"),
                                  G_CALLBACK (action_close_all_windows_callback) },
-  /* name, stock id, label */  { NAUTILUS_ACTION_BACK, GTK_STOCK_GO_BACK, N_("_Back"),
+  /* name, stock id, label */  { NAUTILUS_ACTION_BACK, "go-previous-symbolic", N_("_Back"),
 				 "<alt>Left", N_("Go to the previous visited location"),
 				 G_CALLBACK (action_back_callback) },
-  /* name, stock id, label */  { NAUTILUS_ACTION_FORWARD, GTK_STOCK_GO_FORWARD, N_("_Forward"),
+  /* name, stock id, label */  { NAUTILUS_ACTION_FORWARD, "go-next-symbolic", N_("_Forward"),
 				 "<alt>Right", N_("Go to the next visited location"),
 				 G_CALLBACK (action_forward_callback) },
   /* name, stock id, label */  { "Go to Location", NULL, N_("_Location..."),
@@ -726,7 +605,7 @@ nautilus_window_create_toolbar_action_group (NautilusWindow *window)
 	action = g_object_new (NAUTILUS_TYPE_NAVIGATION_ACTION,
 			       "name", NAUTILUS_ACTION_BACK,
 			       "label", _("_Back"),
-			       "stock_id", GTK_STOCK_GO_BACK,
+			       "icon-name", "go-previous-symbolic",
 			       "tooltip", _("Go to the previous visited location"),
 			       "arrow-tooltip", _("Back history"),
 			       "window", window,
@@ -742,7 +621,7 @@ nautilus_window_create_toolbar_action_group (NautilusWindow *window)
 	action = g_object_new (NAUTILUS_TYPE_NAVIGATION_ACTION,
 			       "name", NAUTILUS_ACTION_FORWARD,
 			       "label", _("_Forward"),
-			       "stock_id", GTK_STOCK_GO_FORWARD,
+			       "icon-name", "go-next-symbolic",
 			       "tooltip", _("Go to the next visited location"),
 			       "arrow-tooltip", _("Forward history"),
 			       "window", window,
@@ -762,8 +641,6 @@ nautilus_window_create_toolbar_action_group (NautilusWindow *window)
 					NULL));
 	gtk_action_group_add_action (action_group, action);
 	gtk_action_set_icon_name (GTK_ACTION (action), "edit-find-symbolic");
-	gtk_action_set_is_important (GTK_ACTION (action), TRUE);
-
 	g_object_unref (action);
 
 	navigation_state = nautilus_window_get_navigation_state (window);
@@ -913,21 +790,11 @@ nautilus_window_initialize_menus (NautilusWindow *window)
 
 	/* add the UI */
 	gtk_ui_manager_add_ui_from_resource (ui_manager, "/org/gnome/nautilus/nautilus-shell-ui.xml", NULL);
-
-	nautilus_window_initialize_trash_icon_monitor (window);
-	nautilus_window_initialize_go_menu (window);
 }
 
 void
 nautilus_window_finalize_menus (NautilusWindow *window)
 {
-	NautilusTrashMonitor *monitor;
-
-	monitor = nautilus_trash_monitor_get ();
-
-	g_signal_handlers_disconnect_by_func (monitor,
-					      trash_state_changed_cb, window);
-
 	g_signal_handlers_disconnect_by_func (nautilus_preferences,
 					      show_hidden_files_preference_callback, window);
 
