@@ -38,8 +38,6 @@
 /* FIXME bugzilla.gnome.org 42575: Migrate more fields into here. */
 struct NautilusWindowDetails
 {
-        GtkWidget *menubar;
-        
         GtkUIManager *ui_manager;
         GtkActionGroup *main_action_group; /* owned by ui_manager */
 
@@ -47,10 +45,6 @@ struct NautilusWindowDetails
         guint extensions_menu_merge_id;
         GtkActionGroup *extensions_menu_action_group;
 
-        GtkActionGroup *bookmarks_action_group;
-        guint bookmarks_merge_id;
-        NautilusBookmarkList *bookmark_list;
-
 	NautilusWindowShowHiddenFilesMode show_hidden_files_mode;
 
 	/* View As menu */
@@ -89,7 +83,6 @@ struct NautilusWindowDetails
         GtkWidget *toolbar;
         GtkActionGroup *toolbar_action_group;
         gboolean temporary_navigation_bar;
-        gboolean temporary_search_bar;
 
         /* focus widget before the location bar has been shown temporarily */
         GtkWidget *last_focus_widget;
