@@ -26,6 +26,7 @@
 
 #include "nautilus-window-slot.h"
 
+#include "nautilus-actions.h"
 #include "nautilus-desktop-window.h"
 #include "nautilus-toolbar.h"
 #include "nautilus-floating-bar.h"
@@ -45,13 +46,95 @@ G_DEFINE_TYPE (NautilusWindowSlot, nautilus_window_slot, GTK_TYPE_BOX);
 enum {
 	ACTIVE,
 	INACTIVE,
+	LOCATION_CHANGED,
 	LAST_SIGNAL
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
+gboolean
+nautilus_window_slot_handle_event (NautilusWindowSlot *slot,
+				   GdkEventKey        *event)
+{
+	return nautilus_query_editor_handle_event (slot->query_editor, event);
+}
+
 static void
-query_editor_changed_callback (NautilusSearchBar *bar,
+sync_search_directory (NautilusWindowSlot *slot)
+{
+	NautilusDirectory *directory;
+	NautilusQuery *query;
+	gchar *text;
+	GFile *location;
+
+	g_assert (NAUTILUS_IS_FILE (slot->viewed_file));
+
+	directory = nautilus_directory_get_for_file (slot->viewed_file);
+	g_assert (NAUTILUS_IS_SEARCH_DIRECTORY (directory));
+
+	query = nautilus_query_editor_get_query (slot->query_editor);
+	text = nautilus_query_get_text (query);
+
+	if (!strlen (text)) {
+		location = nautilus_query_editor_get_location (slot->query_editor);
+		slot->load_with_search = TRUE;
+		nautilus_window_slot_open_location (slot, location, 0);
+		g_object_unref (location);
+	} else {
+		nautilus_search_directory_set_query (NAUTILUS_SEARCH_DIRECTORY (directory),
+						     query);
+		nautilus_window_slot_reload (slot);
+	}
+
+	g_free (text);
+	g_object_unref (query);
+	nautilus_directory_unref (directory);
+}
+
+static void
+sync_search_location_cb (NautilusWindow *window,
+			 GError *error,
+			 gpointer user_data)
+{
+	NautilusWindowSlot *slot = user_data;
+
+	sync_search_directory (slot);
+}
+
+static void
+create_new_search (NautilusWindowSlot *slot)
+{
+	char *uri;
+	NautilusDirectory *directory;
+	GFile *location;
+
+	uri = nautilus_search_directory_generate_new_uri ();
+	location = g_file_new_for_uri (uri);
+
+	directory = nautilus_directory_get (location);
+	g_assert (NAUTILUS_IS_SEARCH_DIRECTORY (directory));
+
+	nautilus_window_slot_open_location_full (slot, location, 0, NULL, sync_search_location_cb, slot);
+
+	nautilus_directory_unref (directory);
+	g_object_unref (location);
+	g_free (uri);
+}
+
+static void
+query_editor_cancel_callback (NautilusQueryEditor *editor,
+			      NautilusWindowSlot *slot)
+{
+	GtkAction *search;
+
+	search = gtk_action_group_get_action (slot->window->details->toolbar_action_group,
+					      NAUTILUS_ACTION_SEARCH);
+
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (search), FALSE);
+}
+
+static void
+query_editor_changed_callback (NautilusQueryEditor *editor,
 			       NautilusQuery *query,
 			       gboolean reload,
 			       NautilusWindowSlot *slot)
@@ -61,75 +144,78 @@ query_editor_changed_callback (NautilusSearchBar *bar,
 	g_assert (NAUTILUS_IS_FILE (slot->viewed_file));
 
 	directory = nautilus_directory_get_for_file (slot->viewed_file);
-	g_assert (NAUTILUS_IS_SEARCH_DIRECTORY (directory));
-
-	nautilus_search_directory_set_query (NAUTILUS_SEARCH_DIRECTORY (directory),
-					     query);
-	if (reload) {
-		nautilus_window_slot_reload (slot);
+	if (!NAUTILUS_IS_SEARCH_DIRECTORY (directory)) {
+		/* this is the first change from the query editor. we
+		   ask for a location change to the search directory,
+		   indicate the directory needs to be sync'd with the
+		   current query. */
+		create_new_search (slot);
+	} else {
+		sync_search_directory (slot);
 	}
 
 	nautilus_directory_unref (directory);
 }
 
 static void
-real_update_query_editor (NautilusWindowSlot *slot)
+update_query_editor (NautilusWindowSlot *slot)
 {
 	NautilusDirectory *directory;
 	NautilusSearchDirectory *search_directory;
-	NautilusQuery *query;
-	GtkWidget *query_editor;
-	gboolean slot_is_active;
-	NautilusWindow *window;
-
-	window = slot->window;
-	query_editor = NULL;
-	slot_is_active = (slot == nautilus_window_get_active_slot (window));
 
 	directory = nautilus_directory_get (slot->location);
+
 	if (NAUTILUS_IS_SEARCH_DIRECTORY (directory)) {
+		NautilusQuery *query;
 		search_directory = NAUTILUS_SEARCH_DIRECTORY (directory);
-
-		if (nautilus_search_directory_is_saved_search (search_directory)) {
-			query_editor = nautilus_query_editor_new (TRUE);
-			nautilus_window_sync_search_widgets (window);
-		} else {
-			GtkWidget *search_bar;
-
-			search_bar = nautilus_toolbar_get_search_bar (NAUTILUS_TOOLBAR (window->details->toolbar));
-			query_editor = nautilus_query_editor_new_with_bar (FALSE,
-									   slot_is_active,
-									   NAUTILUS_SEARCH_BAR (search_bar),
-									   slot);
-		}
-	}
-
-	slot->query_editor = NAUTILUS_QUERY_EDITOR (query_editor);
-
-	if (query_editor != NULL) {
-		g_signal_connect_object (query_editor, "changed",
-					 G_CALLBACK (query_editor_changed_callback), slot, 0);
-		
 		query = nautilus_search_directory_get_query (search_directory);
 		if (query != NULL) {
-			nautilus_query_editor_set_query (NAUTILUS_QUERY_EDITOR (query_editor),
+			nautilus_query_editor_set_query (slot->query_editor,
 							 query);
 			g_object_unref (query);
-		} else {
-			nautilus_query_editor_set_default_query (NAUTILUS_QUERY_EDITOR (query_editor));
 		}
-
-		nautilus_window_slot_add_extra_location_widget (slot, query_editor);
-		gtk_widget_show (query_editor);
-		nautilus_query_editor_grab_focus (NAUTILUS_QUERY_EDITOR (query_editor));
-
-		g_object_add_weak_pointer (G_OBJECT (slot->query_editor),
-					   (gpointer *) &slot->query_editor);
+	} else {
+		nautilus_query_editor_set_location (slot->query_editor, slot->location);
 	}
 
 	nautilus_directory_unref (directory);
 }
 
+static void
+ensure_query_editor (NautilusWindowSlot *slot)
+{
+	g_assert (slot->query_editor != NULL);
+
+	update_query_editor (slot);
+
+	gtk_widget_show (GTK_WIDGET (slot->query_editor));
+	gtk_widget_grab_focus (GTK_WIDGET (slot->query_editor));
+}
+
+void
+nautilus_window_slot_set_query_editor_visible (NautilusWindowSlot *slot,
+					       gboolean            visible)
+{
+	if (visible) {
+		ensure_query_editor (slot);
+
+		if (slot->qe_changed_id == 0)
+			slot->qe_changed_id = g_signal_connect (slot->query_editor, "changed",
+								G_CALLBACK (query_editor_changed_callback), slot);
+		if (slot->qe_cancel_id == 0)
+			slot->qe_cancel_id = g_signal_connect (slot->query_editor, "cancel",
+							       G_CALLBACK (query_editor_cancel_callback), slot);
+
+	} else {
+		gtk_widget_hide (GTK_WIDGET (slot->query_editor));
+		g_signal_handler_disconnect (slot->query_editor, slot->qe_changed_id);
+		slot->qe_changed_id = 0;
+		g_signal_handler_disconnect (slot->query_editor, slot->qe_cancel_id);
+		slot->qe_cancel_id = 0;
+		nautilus_query_editor_set_query (slot->query_editor, NULL);
+	}
+}
+
 static void
 real_active (NautilusWindowSlot *slot)
 {
@@ -182,12 +268,18 @@ nautilus_window_slot_init (NautilusWindowSlot *slot)
 
 	gtk_orientable_set_orientation (GTK_ORIENTABLE (slot),
 					GTK_ORIENTATION_VERTICAL);
+	gtk_widget_show (GTK_WIDGET (slot));
 
 	extras_vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
 	slot->extra_location_widgets = extras_vbox;
 	gtk_box_pack_start (GTK_BOX (slot), extras_vbox, FALSE, FALSE, 0);
 	gtk_widget_show (extras_vbox);
 
+	slot->query_editor = NAUTILUS_QUERY_EDITOR (nautilus_query_editor_new ());
+	nautilus_window_slot_add_extra_location_widget (slot, GTK_WIDGET (slot->query_editor));
+	g_object_add_weak_pointer (G_OBJECT (slot->query_editor),
+				   (gpointer *) &slot->query_editor);
+
 	slot->view_overlay = gtk_overlay_new ();
 	gtk_widget_add_events (slot->view_overlay,
 			       GDK_ENTER_NOTIFY_MASK |
@@ -302,6 +394,16 @@ nautilus_window_slot_class_init (NautilusWindowSlotClass *klass)
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
+	signals[LOCATION_CHANGED] =
+		g_signal_new ("location-changed",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      0,
+			      NULL, NULL,
+			      g_cclosure_marshal_generic,
+			      G_TYPE_NONE, 2,
+			      G_TYPE_STRING,
+			      G_TYPE_STRING);
 }
 
 GFile *
@@ -566,41 +668,27 @@ nautilus_window_slot_set_status (NautilusWindowSlot *slot,
 	}
 }
 
-/* nautilus_window_slot_update_query_editor:
- * 
- * Update the query editor.
- * Called when the location has changed.
- *
- * @slot: The NautilusWindowSlot in question.
- */
-void
-nautilus_window_slot_update_query_editor (NautilusWindowSlot *slot)
+static void
+remove_all_extra_location_widgets (GtkWidget *widget,
+				   gpointer data)
 {
-	if (slot->query_editor != NULL) {
-		gtk_widget_destroy (GTK_WIDGET (slot->query_editor));
-		g_assert (slot->query_editor == NULL);
+	NautilusWindowSlot *slot = data;
+	NautilusDirectory *directory;
+
+	directory = nautilus_directory_get (slot->location);
+	if (widget != GTK_WIDGET (slot->query_editor)) {
+		gtk_container_remove (GTK_CONTAINER (slot->extra_location_widgets), widget);
 	}
 
-	real_update_query_editor (slot);
-}
-
-static void
-remove_all (GtkWidget *widget,
-	    gpointer data)
-{
-	GtkContainer *container;
-	container = GTK_CONTAINER (data);
-
-	gtk_container_remove (container, widget);
+	nautilus_directory_unref (directory);
 }
 
 void
 nautilus_window_slot_remove_extra_location_widgets (NautilusWindowSlot *slot)
 {
 	gtk_container_foreach (GTK_CONTAINER (slot->extra_location_widgets),
-			       remove_all,
-			       slot->extra_location_widgets);
-	gtk_widget_hide (slot->extra_location_widgets);
+			       remove_all_extra_location_widgets,
+			       slot);
 }
 
 void
