@@ -90,6 +90,8 @@ struct NautilusWindowSlot {
 	gboolean allow_stop;
 
 	NautilusQueryEditor *query_editor;
+	gulong qe_changed_id;
+	gulong qe_cancel_id;
 
 	/* New location. */
 	NautilusLocationChangeType location_change_type;
@@ -103,9 +105,12 @@ struct NautilusWindowSlot {
 	gboolean tried_mount;
 	NautilusWindowGoToCallback open_callback;
 	gpointer open_callback_user_data;
+	gboolean load_with_search;
 
 	GCancellable *find_mount_cancellable;
 
+	gboolean visible;
+
 	/* Back/Forward chain, and history list. 
 	 * The data in these lists are NautilusBookmark pointers. 
 	 */
@@ -118,7 +123,10 @@ NautilusWindowSlot * nautilus_window_slot_new (NautilusWindow *window);
 
 void    nautilus_window_slot_update_title		   (NautilusWindowSlot *slot);
 void    nautilus_window_slot_update_icon		   (NautilusWindowSlot *slot);
-void    nautilus_window_slot_update_query_editor	   (NautilusWindowSlot *slot);
+void    nautilus_window_slot_set_query_editor_visible	   (NautilusWindowSlot *slot,
+							    gboolean            visible);
+gboolean nautilus_window_slot_handle_event       	   (NautilusWindowSlot *slot,
+							    GdkEventKey        *event);
 
 GFile * nautilus_window_slot_get_location		   (NautilusWindowSlot *slot);
 char *  nautilus_window_slot_get_location_uri		   (NautilusWindowSlot *slot);
