@@ -38,11 +38,9 @@
 #include "nautilus-notebook.h"
 #include "nautilus-places-sidebar.h"
 #include "nautilus-pathbar.h"
-#include "nautilus-search-bar.h"
 #include "nautilus-toolbar.h"
 #include "nautilus-view-factory.h"
 #include "nautilus-window-manage-views.h"
-#include "nautilus-window-bookmarks.h"
 #include "nautilus-window-slot.h"
 
 #include <eel/eel-debug.h>
@@ -71,7 +69,7 @@
 #include <sys/time.h>
 
 /* dock items */
-#define NAUTILUS_MENU_PATH_SHORT_LIST_PLACEHOLDER  	"/MenuBar/View/View Choices/Short List"
+#define NAUTILUS_MENU_PATH_SHORT_LIST_PLACEHOLDER  	"/ViewMenu/View/View Choices/Short List"
 
 #define MAX_TITLE_LENGTH 180
 
@@ -104,6 +102,8 @@ enum {
 	PROMPT_FOR_LOCATION,
 	LOADING_URI,
 	HIDDEN_FILES_MODE_CHANGED,
+	SLOT_ADDED,
+	SLOT_REMOVED,
 	LAST_SIGNAL
 };
 
@@ -195,85 +195,6 @@ bookmark_list_get_uri_index (GList *list, GFile *location)
 	return -1;
 }
 
-static void
-search_bar_activate_callback (NautilusSearchBar *bar,
-			      NautilusWindow    *window)
-{
-	char *uri, *current_uri;
-	NautilusDirectory *directory;
-	NautilusSearchDirectory *search_directory;
-	NautilusQuery *query;
-	GFile *location;
-	GtkWidget *search_bar;
-
-	uri = nautilus_search_directory_generate_new_uri ();
-	location = g_file_new_for_uri (uri);
-
-	directory = nautilus_directory_get (location);
-	g_assert (NAUTILUS_IS_SEARCH_DIRECTORY (directory));
-
-	search_directory = NAUTILUS_SEARCH_DIRECTORY (directory);
-
-	search_bar = nautilus_toolbar_get_search_bar (NAUTILUS_TOOLBAR (window->details->toolbar));
-
-	query = nautilus_search_bar_get_query (NAUTILUS_SEARCH_BAR (search_bar));
-
-	if (query != NULL) {
-		current_uri = nautilus_window_slot_get_location_uri (window->details->active_slot);
-
-		nautilus_query_set_location (query, current_uri);
-		nautilus_search_directory_set_query (search_directory, query);
-
-		g_free (current_uri);
-		g_object_unref (query);
-	}
-
-	nautilus_window_slot_open_location (window->details->active_slot, location, 0);
-
-	nautilus_directory_unref (directory);
-	g_object_unref (location);
-	g_free (uri);
-}
-
-static void
-nautilus_window_hide_temporary_bars (NautilusWindow *window)
-{
-	NautilusWindowSlot *slot;
-	NautilusDirectory *directory;
-
-	slot = window->details->active_slot;
-
-	if (window->details->temporary_navigation_bar) {
-		directory = nautilus_directory_get (slot->location);
-
-		window->details->temporary_navigation_bar = FALSE;
-
-		/* if we're in a search directory, hide the main bar, and show the search
-		 * bar again; otherwise, just hide the whole toolbar.
-		 */
-		if (NAUTILUS_IS_SEARCH_DIRECTORY (directory)) {
-			nautilus_toolbar_set_show_main_bar (NAUTILUS_TOOLBAR (window->details->toolbar), FALSE);
-			nautilus_toolbar_set_show_search_bar (NAUTILUS_TOOLBAR (window->details->toolbar), TRUE);
-		} else {
-			gtk_widget_hide (window->details->toolbar);
-		}
-
-		nautilus_directory_unref (directory);
-	}
-}
-
-static void
-search_bar_cancel_callback (GtkWidget      *widget,
-			    NautilusWindow *window)
-{
-	GtkAction *search;
-
-	search = gtk_action_group_get_action (window->details->toolbar_action_group,
-					      NAUTILUS_ACTION_SEARCH);
-
-	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (search), FALSE);
-}
-
 static void
 unset_focus_widget (NautilusWindow *window)
 {
@@ -284,29 +205,13 @@ unset_focus_widget (NautilusWindow *window)
 	}
 }
 
-static gboolean
-widget_is_in_temporary_bars (GtkWidget      *widget,
-			     NautilusWindow *window)
-{
-	gboolean res = FALSE;
-
-	if ((gtk_widget_get_ancestor (widget, NAUTILUS_TYPE_LOCATION_BAR) != NULL &&
-	     window->details->temporary_navigation_bar) ||
-	    (gtk_widget_get_ancestor (widget, NAUTILUS_TYPE_SEARCH_BAR) != NULL &&
-	     window->details->temporary_search_bar))
-		res = TRUE;
-
-	return res;
-}
-
 static void
 remember_focus_widget (NautilusWindow *window)
 {
 	GtkWidget *focus_widget;
 
 	focus_widget = gtk_window_get_focus (GTK_WINDOW (window));
-	if (focus_widget != NULL &&
-	    !widget_is_in_temporary_bars (focus_widget, window)) {
+	if (focus_widget != NULL) {
 		unset_focus_widget (window);
 
 		window->details->last_focus_widget = focus_widget;
@@ -343,42 +248,15 @@ navigation_bar_cancel_callback (GtkWidget      *widget,
 {
 	nautilus_toolbar_set_show_location_entry (NAUTILUS_TOOLBAR (window->details->toolbar), FALSE);
 
-	nautilus_window_hide_temporary_bars (window);
 	restore_focus_widget (window);
 }
 
-static void
-nautilus_window_ensure_search_bar (NautilusWindow *window)
-{
-	GtkWidget *search_bar;
-
-	remember_focus_widget (window);
-	nautilus_toolbar_set_show_search_bar (NAUTILUS_TOOLBAR (window->details->toolbar), TRUE);
-	search_bar = nautilus_toolbar_get_search_bar (NAUTILUS_TOOLBAR (window->details->toolbar));
-	nautilus_search_bar_grab_focus (NAUTILUS_SEARCH_BAR (search_bar));
-}
-
-static void
-nautilus_window_hide_search_bar (NautilusWindow *window)
-{
-	nautilus_toolbar_set_show_search_bar (NAUTILUS_TOOLBAR (window->details->toolbar), FALSE);
-	restore_focus_widget (window);
-
-	if (window->details->temporary_search_bar) {
-		window->details->temporary_search_bar = FALSE;
-
-		gtk_widget_hide (window->details->toolbar);
-	}
-}
-
 static void
 navigation_bar_location_changed_callback (GtkWidget      *widget,
 					  GFile          *location,
 					  NautilusWindow *window)
 {
 	nautilus_toolbar_set_show_location_entry (NAUTILUS_TOOLBAR (window->details->toolbar), FALSE);
-	nautilus_window_hide_search_bar (window);
-	nautilus_window_hide_temporary_bars (window);
 
 	restore_focus_widget (window);
 
@@ -412,31 +290,21 @@ action_show_hide_search_callback (GtkAction *action,
 				  gpointer user_data)
 {
 	NautilusWindow *window = user_data;
+	NautilusWindowSlot *slot;
+
+	slot = window->details->active_slot;
 
 	if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action))) {
-		nautilus_window_ensure_search_bar (window);
+		remember_focus_widget (window);
+		nautilus_window_slot_set_query_editor_visible (slot, TRUE);
 	} else {
-		NautilusWindowSlot *slot;
 		GFile *location = NULL;
 
-		slot = window->details->active_slot;
-		nautilus_window_hide_search_bar (window);
+		restore_focus_widget (window);
 
 		/* Use the location bar as the return location */
 		if (slot->query_editor != NULL) {
-			NautilusQuery *query;
-			char *uri;
-
-			query = nautilus_query_editor_get_query (slot->query_editor);
-			if (query != NULL) {
-				uri = nautilus_query_get_location (query);
-				if (uri != NULL) {
-					location = g_file_new_for_uri (uri);
-					g_free (uri);
-				}
-				g_object_unref (query);
-			}
-
+			location = nautilus_query_editor_get_location (slot->query_editor);
 			/* Last try: use the home directory as the return location */
 			if (location == NULL) {
 				location = g_file_new_for_path (g_get_home_dir ());
@@ -445,6 +313,8 @@ action_show_hide_search_callback (GtkAction *action,
 			nautilus_window_go_to (window, location);
 			g_object_unref (location);
 		}
+
+		nautilus_window_slot_set_query_editor_visible (slot, FALSE);
 	}
 }
 
@@ -460,6 +330,9 @@ close_slot (NautilusWindow     *window,
 	DEBUG ("Closing slot %p", slot);
 
 	nautilus_window_manage_views_close_slot (slot);
+	window->details->slots = g_list_remove (window->details->slots, slot);
+
+	g_signal_emit (window, signals[SLOT_REMOVED], 0, slot);
 
 	notebook = GTK_NOTEBOOK (window->details->notebook);
 	page_num = gtk_notebook_page_num (notebook, GTK_WIDGET (slot));
@@ -476,7 +349,6 @@ close_slot (NautilusWindow     *window,
 
 	gtk_notebook_set_show_tabs (notebook,
 				    gtk_notebook_get_n_pages (notebook) > 1);
-	window->details->slots = g_list_remove (window->details->slots, slot);
 }
 
 NautilusWindowSlot *
@@ -503,6 +375,7 @@ nautilus_window_open_slot (NautilusWindow             *window,
 					   window);
 
 	window->details->slots = g_list_append (window->details->slots, slot);
+	g_signal_emit (window, signals[SLOT_ADDED], 0, slot);
 
 	return slot;
 }
@@ -563,19 +436,23 @@ void
 nautilus_window_sync_allow_stop (NautilusWindow *window,
 				 NautilusWindowSlot *slot)
 {
-	GtkAction *action;
+	GtkAction *stop_action;
+	GtkAction *reload_action;
 	gboolean allow_stop, slot_is_active;
 
-	action = gtk_action_group_get_action (nautilus_window_get_main_action_group (window),
-					      NAUTILUS_ACTION_STOP);
-	allow_stop = gtk_action_get_sensitive (action);
+	stop_action = gtk_action_group_get_action (nautilus_window_get_main_action_group (window),
+						   NAUTILUS_ACTION_STOP);
+	reload_action = gtk_action_group_get_action (nautilus_window_get_main_action_group (window),
+						     NAUTILUS_ACTION_RELOAD);
+	allow_stop = gtk_action_get_sensitive (stop_action);
 
 	slot_is_active = (slot == nautilus_window_get_active_slot (window));
 
 	if (!slot_is_active ||
 	    allow_stop != slot->allow_stop) {
 		if (slot_is_active) {
-			gtk_action_set_sensitive (action, slot->allow_stop);
+			gtk_action_set_visible (stop_action, slot->allow_stop);
+			gtk_action_set_visible (reload_action, !slot->allow_stop);
 		}
 
 		if (gtk_widget_get_realized (GTK_WIDGET (window))) {
@@ -820,17 +697,14 @@ nautilus_window_slot_close (NautilusWindow     *window,
 }
 
 static void
-toggle_toolbar_search_button (NautilusWindow *window)
+toggle_toolbar_search_button (NautilusWindow *window,
+			      gboolean        active)
 {
 	GtkAction *action;
 
 	action = gtk_action_group_get_action (window->details->toolbar_action_group, NAUTILUS_ACTION_SEARCH);
 
-	g_signal_handlers_block_by_func (action,
-					 action_show_hide_search_callback, window);
-	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
-	g_signal_handlers_unblock_by_func (action,
-					   action_show_hide_search_callback, window);
+	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), active);
 }
 
 void
@@ -838,26 +712,21 @@ nautilus_window_sync_search_widgets (NautilusWindow *window)
 {
 	NautilusDirectory *directory;
 	NautilusSearchDirectory *search_directory;
+	NautilusWindowSlot *slot;
 
 	search_directory = NULL;
+	slot = window->details->active_slot;
 
-	directory = nautilus_directory_get (window->details->active_slot->location);
+	directory = nautilus_directory_get (slot->location);
 	if (NAUTILUS_IS_SEARCH_DIRECTORY (directory)) {
 		search_directory = NAUTILUS_SEARCH_DIRECTORY (directory);
 	}
 
-	if (search_directory != NULL) {
-		if (!nautilus_search_directory_is_saved_search (search_directory)) {
-			nautilus_toolbar_set_show_search_bar (NAUTILUS_TOOLBAR (window->details->toolbar), TRUE);
-			window->details->temporary_search_bar = FALSE;
-		} else {
-			toggle_toolbar_search_button (window);
-		}
+	if (search_directory != NULL || slot->load_with_search) {
+		slot->load_with_search = FALSE;
+		toggle_toolbar_search_button (window, TRUE);
 	} else {
-		GtkWidget *search_bar;
-
-		search_bar = nautilus_toolbar_get_search_bar (NAUTILUS_TOOLBAR (window->details->toolbar));
-		search_bar_cancel_callback (search_bar, window);
+		toggle_toolbar_search_button (window, FALSE);
 	}
 
 	nautilus_directory_unref (directory);
@@ -871,8 +740,6 @@ nautilus_window_sync_location_widgets (NautilusWindow *window)
 
 	slot = window->details->active_slot;
 
-	nautilus_window_hide_temporary_bars (window);
-
 	/* Change the location bar and path bar to match the current location. */
 	if (slot->location != NULL) {
 		char *uri;
@@ -910,7 +777,6 @@ nautilus_window_ensure_location_bar (NautilusWindow *window)
 
 	remember_focus_widget (window);
 
-	nautilus_toolbar_set_show_main_bar (NAUTILUS_TOOLBAR (window->details->toolbar), TRUE);
 	nautilus_toolbar_set_show_location_entry (NAUTILUS_TOOLBAR (window->details->toolbar), TRUE);
 
 	location_bar = nautilus_toolbar_get_location_bar (NAUTILUS_TOOLBAR (window->details->toolbar));
@@ -1128,7 +994,6 @@ create_toolbar (NautilusWindow *window)
 	GtkWidget *toolbar;
 	GtkWidget *path_bar;
 	GtkWidget *location_bar;
-	GtkWidget *search_bar;
 
 	header_size_group = gtk_size_group_new (GTK_SIZE_GROUP_VERTICAL);
 	gtk_size_group_set_ignore_hidden (header_size_group, FALSE);
@@ -1141,11 +1006,7 @@ create_toolbar (NautilusWindow *window)
 			  G_CALLBACK (action_show_hide_search_callback), window);
 	nautilus_navigation_state_set_master (window->details->nav_state, window->details->toolbar_action_group );
 
-	toolbar = nautilus_toolbar_new (action_group);
-	gtk_box_pack_start (GTK_BOX (window->details->main_view),
-			    toolbar,
-			    FALSE, FALSE, 0);
-	gtk_widget_show (toolbar);
+	toolbar = nautilus_toolbar_new (nautilus_window_get_ui_manager (NAUTILUS_WINDOW (window)), action_group);
 
 	g_object_bind_property (window, "disable-chrome",
 				toolbar, "visible",
@@ -1174,15 +1035,6 @@ create_toolbar (NautilusWindow *window)
 	g_signal_connect_object (location_bar, "cancel",
 				 G_CALLBACK (navigation_bar_cancel_callback), window, 0);
 
-	/* connect to the search bar signals */
-	search_bar = nautilus_toolbar_get_search_bar (NAUTILUS_TOOLBAR (toolbar));
-	gtk_size_group_add_widget (header_size_group, search_bar);
-
-	g_signal_connect_object (search_bar, "activate",
-				 G_CALLBACK (search_bar_activate_callback), window, 0);
-	g_signal_connect_object (search_bar, "cancel",
-				 G_CALLBACK (search_bar_cancel_callback), window, 0);
-
 	g_object_unref (header_size_group);
 
 	return toolbar;
@@ -1224,7 +1076,6 @@ nautilus_window_constructed (GObject *self)
 {
 	NautilusWindow *window;
 	GtkWidget *grid;
-	GtkWidget *menu;
 	NautilusWindowSlot *slot;
 
 	window = NAUTILUS_WINDOW (self);
@@ -1244,15 +1095,14 @@ nautilus_window_constructed (GObject *self)
 	nautilus_window_initialize_menus (window);
 	nautilus_window_initialize_actions (window);
 
-	menu = gtk_ui_manager_get_widget (window->details->ui_manager, "/MenuBar");
-	window->details->menubar = menu;
-	gtk_widget_set_hexpand (menu, TRUE);
-	gtk_widget_show (menu);
-	gtk_container_add (GTK_CONTAINER (grid), menu);
-
 	/* Register to menu provider extension signal managing menu updates */
 	g_signal_connect_object (nautilus_signaller_get_current (), "popup_menu_changed",
 			 G_CALLBACK (nautilus_window_load_extension_menus), window, G_CONNECT_SWAPPED);
+	window->details->toolbar = create_toolbar (window);
+	gtk_container_add (GTK_CONTAINER (grid), window->details->toolbar);
+	gtk_widget_set_hexpand (window->details->toolbar, TRUE);
+	gtk_widget_set_vexpand (window->details->toolbar, FALSE);
+	gtk_widget_show (window->details->toolbar);
 
 	window->details->content_paned = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
 	gtk_widget_set_hexpand (window->details->content_paned, TRUE);
@@ -1266,10 +1116,7 @@ nautilus_window_constructed (GObject *self)
 			 TRUE, FALSE);
 	gtk_widget_show (window->details->main_view);
 
-	window->details->toolbar = create_toolbar (window);
 	window->details->notebook = create_notebook (window);
-
-	nautilus_window_initialize_bookmarks_menu (window);
 	nautilus_window_set_initial_window_geometry (window);
 
 	slot = nautilus_window_open_slot (window, 0);
@@ -1371,7 +1218,6 @@ nautilus_window_finalize (GObject *object)
 	nautilus_window_finalize_menus (window);
 
 	g_clear_object (&window->details->nav_state);
-	g_clear_object (&window->details->bookmark_list);
 	g_clear_object (&window->details->ui_manager);
 
 	free_stored_viewers (window);
@@ -1393,19 +1239,24 @@ nautilus_window_view_visible (NautilusWindow *window,
 
 	slot = nautilus_window_get_slot_for_view (window, view);
 
-	if (gtk_widget_get_visible (GTK_WIDGET (slot))) {
+	if (slot->visible) {
 		return;
 	}
 
-	gtk_widget_show (GTK_WIDGET (slot));
+	slot->visible = TRUE;
 
 	/* Look for other non-visible slots */
 	for (l = window->details->slots; l != NULL; l = l->next) {
 		slot = l->data;
 
-		if (!gtk_widget_get_visible (GTK_WIDGET (slot))) {
+		if (!slot->visible) {
 			return;
 		}
+	}
+
+	/* Look for other non-visible slots */
+	for (l = window->details->slots; l != NULL; l = l->next) {
+		slot = l->data;
 
 		nautilus_window_slot_update_title (slot);
 		nautilus_window_slot_update_icon (slot);
@@ -1603,6 +1454,14 @@ nautilus_window_key_press_event (GtkWidget *widget,
 		}
 	}
 
+	if (event->keyval != GDK_KEY_slash /* don't steal slash key event, used for "go to" */ &&
+	    event->keyval != GDK_KEY_Delete) {
+		if (nautilus_window_slot_handle_event (window->details->active_slot, event)) {
+			toggle_toolbar_search_button (window, TRUE);
+			return TRUE;
+		}
+	}
+
 	return GTK_WIDGET_CLASS (nautilus_window_parent_class)->key_press_event (widget, event);
 }
 
@@ -2156,6 +2015,14 @@ nautilus_window_get_active_slot (NautilusWindow *window)
 	return window->details->active_slot;
 }
 
+GList *
+nautilus_window_get_slots (NautilusWindow *window)
+{
+	g_assert (NAUTILUS_IS_WINDOW (window));
+
+	return window->details->slots;
+}
+
 static void
 nautilus_window_reload (NautilusWindow *window)
 {
@@ -2254,6 +2121,8 @@ use_extra_mouse_buttons_changed (gpointer callback_data)
 static void
 nautilus_window_init (NautilusWindow *window)
 {
+	GtkWindowGroup *window_group;
+
 	window->details = G_TYPE_INSTANCE_GET_PRIVATE (window, NAUTILUS_TYPE_WINDOW, NautilusWindowDetails);
 
 	window->details->slots = NULL;
@@ -2261,8 +2130,13 @@ nautilus_window_init (NautilusWindow *window)
 
 	window->details->show_hidden_files_mode = NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_DEFAULT;
 
+	window_group = gtk_window_group_new ();
+	gtk_window_group_add_window (window_group, GTK_WINDOW (window));
+	g_object_unref (window_group);
+
 	/* Set initial window title */
 	gtk_window_set_title (GTK_WINDOW (window), _("Files"));
+	gtk_window_set_hide_titlebar_when_maximized (GTK_WINDOW (window), TRUE);
 }
 
 static NautilusIconInfo *
@@ -2358,10 +2232,24 @@ nautilus_window_class_init (NautilusWindowClass *class)
 			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_STRING);
+	signals[SLOT_ADDED] =
+		g_signal_new ("slot-added",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+			      0,
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE, 1, NAUTILUS_TYPE_WINDOW_SLOT);
+	signals[SLOT_REMOVED] =
+		g_signal_new ("slot-removed",
+			      G_TYPE_FROM_CLASS (class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+			      0,
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE, 1, NAUTILUS_TYPE_WINDOW_SLOT);
 
 	binding_set = gtk_binding_set_by_class (class);
-	gtk_binding_entry_add_signal (binding_set, GDK_KEY_BackSpace, 0,
-				      "go-up", 0);
 	gtk_binding_entry_add_signal (binding_set, GDK_KEY_F5, 0,
 				      "reload", 0);
 	gtk_binding_entry_add_signal (binding_set, GDK_KEY_slash, 0,
